#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the SkyDriveLog log parser."""

from __future__ import unicode_literals

import unittest

from plaso.formatters import skydrivelog as _  # pylint: disable=unused-import
from plaso.parsers import skydrivelog

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class SkyDriveLogUnitTest(test_lib.ParserTestCase):
  """Tests for the SkyDrive log parser."""

  @shared_test_lib.skipUnlessHasTestFile(['skydriveerr.log'])
  def testParseErrorLog(self):
    """Tests the Parse function or error log."""
    parser = skydrivelog.SkyDriveLogParser()
    storage_writer = self._ParseFile(['skydriveerr.log'], parser)

    self.assertEqual(storage_writer.number_of_events, 19)

    events = list(storage_writer.GetEvents())

    event = events[0]

    self.CheckTimestamp(event.timestamp, '2013-07-25 16:03:23.291000')

    expected_message = (
        'Logging started. Version= 17.0.2011.0627 StartLocalTime: '
        '2013-07-25-180323.291 PID=0x8f4 TID=0x718 ContinuedFrom=')
    expected_short_message = (
        'Logging started. Version= 17.0.2011.0627 StartLocalTime: '
        '2013-07-25-180323.29...')
    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    event = events[1]

    self.CheckTimestamp(event.timestamp, '2013-07-25 16:03:24.649000')

    expected_message = (
        '[AUTH authapi.cpp(280) ERR] Sign in failed : '
        'DRX_E_AUTH_NO_VALID_CREDENTIALS,')
    expected_short_message = (
        'Sign in failed : DRX_E_AUTH_NO_VALID_CREDENTIALS,')
    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    event = events[18]

    self.CheckTimestamp(event.timestamp, '2013-08-01 21:27:44.124000')

    expected_message = (
        '[WNS absconn.cpp(177) VRB] Received data from server,'
        'dwID=0x0;dwSize=0x3e;pbData=PNG 9 CON 48  <ping-response>'
        '<wait>44</wait></ping-response>')
    expected_short_message = (
        'Received data from server,dwID=0x0;dwSize=0x3e;pbData=PNG 9 CON 48  '
        '<ping-res...')
    self._TestGetMessageStrings(event, expected_message, expected_short_message)

  @shared_test_lib.skipUnlessHasTestFile(['skydriveerr-unicode.log'])
  def testParseErrorLogUnicode(self):
    """Tests the Parse function on Unicode error log."""
    parser = skydrivelog.SkyDriveLogParser()
    storage_writer = self._ParseFile(
        ['skydriveerr-unicode.log'], parser)

    self.assertEqual(storage_writer.number_of_events, 19)

    events = list(storage_writer.GetEvents())

    # TODO: check if this test passes because the encoding on my system
    # is UTF-8.
    event = events[3]

    expected_detail = (
        'No node found named Passport-Jméno-člena, no user name available,')
    self.assertEqual(event.detail, expected_detail)

  @shared_test_lib.skipUnlessHasTestFile(['skydrive.log'])
  def testParseLog(self):
    """Tests the Parse function on normal log."""
    parser = skydrivelog.SkyDriveLogParser()
    storage_writer = self._ParseFile(['skydrive.log'], parser)

    self.assertEqual(storage_writer.number_of_events, 17)

    events = list(storage_writer.GetEvents())

    event = events[0]

    self.CheckTimestamp(event.timestamp, '2013-08-12 01:08:52.985000')

    event = events[1]

    self.CheckTimestamp(event.timestamp, '2013-08-12 01:10:08.835000')

    event = events[11]

    self.CheckTimestamp(event.timestamp, '2013-08-12 02:52:32.976000')

    expected_message = (
        '[WNS absconn.cpp(177) VRB] Received data from server,dwID=0x0;'
        'dwSize=0x15a;pbData=GET 5 WNS 331 Context: 2891  <channel-response>'
        '<id>1;13714367258539257282</id><exp>2013-09-11T02:52:37Z</exp><url>'
        'https://bn1.notify.windows.com/?token=AgYAAAAdkHjSxiNH1mbF0Rp5TIv0K'
        'z317BKYIAfBNO6szULCOEE2393owBINnPC5xoika5SJlNtXZ%2bwzaRVsPRcP1p64XF'
        'n90vGwr07DGZxfna%2bxBpBBplzZhLV9y%2fNV%2bBPxNmTI5sRgaZ%2foGvYCIj6Md'
        'eU1</url></channel-response>')
    expected_short_message = (
        'Received data from server,dwID=0x0;dwSize=0x15a;pbData=GET 5 WNS '
        '331 Context:...')
    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    event = events[13]

    self.CheckTimestamp(event.timestamp, '2013-08-12 03:18:57.232000')

    expected_message = (
        'Logging started. Version= 17.0.2011.0627 StartLocalTime: '
        '2013-08-11-231857.232 PID=0x1ef0 TID=0x1ef4 ContinuedFrom=')
    expected_short_message = (
        'Logging started. Version= 17.0.2011.0627 StartLocalTime: '
        '2013-08-11-231857.23...')
    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    event = events[15]

    self.CheckTimestamp(event.timestamp, '2013-08-31 03:45:37.940000')

    expected_message = (
        '[PAL cwinhttp.cpp(1581) VRB] ,output=GET <- /MyData/LiveFolders?'
        'Filter=changes&InlineBlobs=false&MaxItemCount=50&SyncToken=LM%3d6351'
        '1875645970%3bID%3d7F095149027848ED!103%3bLR%3d63513517536493%3bEP%3d'
        '2%3bTD%3dTrue&View=SkyDriveSync;m_httpStatus=0x130;hr=8004db30;'
        'm_pSink=null;cb=0x0;msec=0x4e')
    expected_short_message = (
        ',output=GET <- /MyData/LiveFolders?Filter=changes&InlineBlobs='
        'false&MaxItemCo...')
    self._TestGetMessageStrings(event, expected_message, expected_short_message)


class SkyDriveOldLogUnitTest(test_lib.ParserTestCase):
  """Tests for the SkyDrive old log parser."""

  @shared_test_lib.skipUnlessHasTestFile(['skydrive_old.log'])
  def testParse(self):
    """Tests the Parse function."""
    parser = skydrivelog.SkyDriveOldLogParser()
    storage_writer = self._ParseFile(['skydrive_old.log'], parser)

    self.assertEqual(storage_writer.number_of_events, 18)

    events = list(storage_writer.GetEvents())

    event = events[0]

    self.CheckTimestamp(event.timestamp, '2013-08-01 21:22:28.999000')

    expected_message = (
        '[global.cpp:626!logVersionInfo] (DETAIL) 17.0.2011.0627 (Ship)')
    expected_short_message = (
        '17.0.2011.0627 (Ship)')
    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    event = events[1]

    self.CheckTimestamp(event.timestamp, '2013-08-01 21:22:29.702000')

    event = events[2]

    self.CheckTimestamp(event.timestamp, '2013-08-01 21:22:29.702000')

    event = events[3]

    self.CheckTimestamp(event.timestamp, '2013-08-01 21:22:29.702000')

    expected_message = (
        'SyncToken = LM%3d12345678905670%3bID%3d1234567890E059C0!'
        '103%3bLR%3d12345678905623%3aEP%3d2')
    expected_short_message = (
        'SyncToken = LM%3d12345678905670%3bID%3d1234567890E059C0!'
        '103%3bLR%3d1234567890...')
    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    event = events[4]

    self.CheckTimestamp(event.timestamp, '2013-08-01 21:22:58.344000')

    event = events[5]

    self.CheckTimestamp(event.timestamp, '2013-08-01 21:22:58.344000')

    event = events[17]

    self.CheckTimestamp(event.timestamp, '2013-08-01 21:28:46.742000')

    expected_message = (
        'SyncToken = Not a sync token (\xe0\xe8\xec\xf2\xf9)!')
    self._TestGetMessageStrings(event, expected_message, expected_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Parser for the Sleuthkit (TSK) bodyfile or mactime format.

The format specifications can be read here:
  http://wiki.sleuthkit.org/index.php?title=Body_file
"""

from __future__ import unicode_literals

import re

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import dsv_parser
from plaso.parsers import manager


# TODO: refactor to pass user_sid as an int.
class MactimeEventData(events.EventData):
  """Mactime event data.

  Attributes:
    filename (str): name of the file.
    inode (int): "inode" of the file. Note that inode is an overloaded term
        in the context of mactime and used for MFT entry index values as well.
    md5_hash (str): MD5 hash of the file content.
    mode_as_string (str): protection mode.
    offset (int): number of the corresponding line.
    size (int): size of the file content.
    user_gid (int): user group identifier (GID).
    user_sid (str): user security identifier (SID).
  """

  DATA_TYPE = 'fs:mactime:line'

  def __init__(self):
    """Initializes event data."""
    super(MactimeEventData, self).__init__(data_type=self.DATA_TYPE)
    self.filename = None
    self.inode = None
    self.md5 = None
    self.mode_as_string = None
    self.offset = None
    self.size = None
    self.user_gid = None
    self.user_sid = None


class MactimeParser(dsv_parser.DSVParser):
  """SleuthKit bodyfile parser."""

  NAME = 'mactime'
  DESCRIPTION = 'Parser for SleuthKit version 3 bodyfiles.'

  COLUMNS = [
      'md5', 'name', 'inode', 'mode_as_string', 'uid', 'gid', 'size',
      'atime', 'mtime', 'ctime', 'btime']
  DELIMITER = b'|'

  _MD5_RE = re.compile(r'^[0-9a-fA-F]{32}$')

  # Mapping according to:
  # http://wiki.sleuthkit.org/index.php?title=Mactime_output
  _TIMESTAMP_DESC_MAP = {
      'atime': definitions.TIME_DESCRIPTION_LAST_ACCESS,
      'btime': definitions.TIME_DESCRIPTION_CREATION,
      'ctime': definitions.TIME_DESCRIPTION_CHANGE,
      'mtime': definitions.TIME_DESCRIPTION_MODIFICATION,
  }

  def _GetIntegerValue(self, row, value_name):
    """Converts a specific value of the row to an integer.

    Args:
      row (dict[str, str]): fields of a single row, as specified in COLUMNS.
      value_name (str): name of the value within the row.

    Returns:
      int: value or None if the value cannot be converted.
    """
    value = row.get(value_name, None)
    try:
      return int(value, 10)
    except (TypeError, ValueError):
      return None

  def ParseRow(self, parser_mediator, row_offset, row):
    """Parses a line of the log file and produces events.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row_offset (int): number of the corresponding line.
      row (dict[str, str]): fields of a single row, as specified in COLUMNS.
    """
    filename = row.get('name', None)
    md5_hash = row.get('md5', None)
    mode = row.get('mode_as_string', None)

    inode_number = row.get('inode', None)
    if '-' in inode_number:
      inode_number, _, _ = inode_number.partition('-')

    try:
      inode_number = int(inode_number, 10)
    except (TypeError, ValueError):
      inode_number = None

    data_size = self._GetIntegerValue(row, 'size')
    user_uid = self._GetIntegerValue(row, 'uid')
    user_gid = self._GetIntegerValue(row, 'gid')

    event_data = MactimeEventData()
    event_data.filename = filename
    event_data.inode = inode_number
    event_data.md5 = md5_hash
    event_data.mode_as_string = mode
    event_data.offset = row_offset
    event_data.size = data_size
    event_data.user_gid = user_gid

    if user_uid is None:
      event_data.user_sid = None
    else:
      # Note that the user_sid value is expected to be a string.
      event_data.user_sid = '{0:d}'.format(user_uid)

    for value_name, timestamp_description in iter(
        self._TIMESTAMP_DESC_MAP.items()):
      posix_time = self._GetIntegerValue(row, value_name)
      # mactime will return 0 if the timestamp is not set.
      if not posix_time:
        continue

      date_time = dfdatetime_posix_time.PosixTime(timestamp=posix_time)
      event = time_events.DateTimeValuesEvent(date_time, timestamp_description)
      parser_mediator.ProduceEventWithEventData(event, event_data)

  def VerifyRow(self, unused_parser_mediator, row):
    """Verifies if a line of the file is in the expected format.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row (dict[str, str]): fields of a single row, as specified in COLUMNS.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """
    # Sleuthkit version 3 format:
    # MD5|name|inode|mode_as_string|UID|GID|size|atime|mtime|ctime|crtime
    # 0|/lost+found|11|d/drwx------|0|0|12288|1337961350|1337961350|1337961350|0

    if row['md5'] != b'0' and not self._MD5_RE.match(row['md5']):
      return False

    # Check if the following columns contain a base 10 integer value if set.
    for column_name in (
        'uid', 'gid', 'size', 'atime', 'mtime', 'ctime', 'crtime'):
      column_value = row.get(column_name, None)
      if not column_value:
        continue

      try:
        int(column_value, 10)
      except (TypeError, ValueError):
        return False

    return True


manager.ParsersManager.RegisterParser(MactimeParser)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the Elastic Search output module CLI arguments helper."""

from __future__ import unicode_literals

import argparse
import unittest

from plaso.cli.helpers import elastic_output
from plaso.lib import errors
from plaso.output import elastic

from tests.cli import test_lib as cli_test_lib
from tests.cli.helpers import test_lib


class ElasticSearchOutputArgumentsHelperTest(
    test_lib.OutputModuleArgumentsHelperTest):
  """Tests the Elastic Search output module CLI arguments helper."""

  # pylint: disable=no-member,protected-access

  _EXPECTED_OUTPUT = """\
usage: cli_helper.py [--index_name INDEX_NAME] [--doc_type DOCUMENT_TYPE]
                     [--flush_interval FLUSH_INTERVAL] [--raw_fields]
                     [--elastic_user ELASTIC_USER] [--server HOSTNAME]
                     [--port PORT]

Test argument parser.

optional arguments:
  --doc_type DOCUMENT_TYPE
                        Name of the document type that will be used in
                        ElasticSearch.
  --elastic_user ELASTIC_USER
                        Username to use for Elasticsearch authentication.
  --flush_interval FLUSH_INTERVAL
                        Events to queue up before bulk insert to
                        ElasticSearch.
  --index_name INDEX_NAME
                        Name of the index in ElasticSearch.
  --port PORT           The port number of the server.
  --raw_fields          Export string fields that will not be analyzed by
                        Lucene.
  --server HOSTNAME     The hostname or server IP address of the server.
"""

  def testAddArguments(self):
    """Tests the AddArguments function."""
    argument_parser = argparse.ArgumentParser(
        prog='cli_helper.py',
        description='Test argument parser.', add_help=False,
        formatter_class=cli_test_lib.SortedArgumentsHelpFormatter)

    elastic_output.ElasticSearchOutputArgumentsHelper.AddArguments(
        argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    options = cli_test_lib.TestOptions()

    output_mediator = self._CreateOutputMediator()
    output_module = elastic.ElasticsearchOutputModule(output_mediator)
    elastic_output.ElasticSearchOutputArgumentsHelper.ParseOptions(
        options, output_module)

    with self.assertRaises(errors.BadConfigObject):
      elastic_output.ElasticSearchOutputArgumentsHelper.ParseOptions(
          options, None)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the USB Windows Registry plugin."""

from __future__ import unicode_literals

import unittest

from plaso.formatters import winreg  # pylint: disable=unused-import
from plaso.parsers.winreg_plugins import usb

from tests import test_lib as shared_test_lib
from tests.parsers.winreg_plugins import test_lib


class USBPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the USB Windows Registry plugin."""

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = usb.USBPlugin()

    key_path = 'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Enum\\USB'
    self._AssertFiltersOnKeyPath(plugin, key_path)

    self._AssertNotFiltersOnKeyPath(plugin, 'HKEY_LOCAL_MACHINE\\Bogus')

  @shared_test_lib.skipUnlessHasTestFile(['SYSTEM'])
  def testProcess(self):
    """Tests the Process function."""
    test_file_entry = self._GetTestFileEntry(['SYSTEM'])
    key_path = 'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Enum\\USB'

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = usb.USBPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    self.assertEqual(storage_writer.number_of_events, 7)

    events = list(storage_writer.GetEvents())

    event = events[3]

    self.assertEqual(event.pathspec, test_file_entry.path_spec)
    # This should just be the plugin name, as we're invoking it directly,
    # and not through the parser.
    self.assertEqual(event.parser, plugin.plugin_name)

    expected_value = 'VID_0E0F&PID_0002'
    self._TestRegvalue(event, 'subkey_name', expected_value)
    self._TestRegvalue(event, 'vendor', 'VID_0E0F')
    self._TestRegvalue(event, 'product', 'PID_0002')

    self.CheckTimestamp(event.timestamp, '2012-04-07 10:31:37.625247')

    expected_message = (
        '[{0:s}] '
        'product: PID_0002 '
        'serial: 6&2ab01149&0&2 '
        'subkey_name: VID_0E0F&PID_0002 '
        'vendor: VID_0E0F').format(key_path)
    expected_short_message = '{0:s}...'.format(expected_message[:77])

    self._TestGetMessageStrings(event, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The event filters CLI arguments helper."""

from __future__ import unicode_literals

from plaso.cli import time_slices
from plaso.cli import tools
from plaso.cli.helpers import interface
from plaso.cli.helpers import manager
from plaso.filters import manager as filters_manager
from plaso.lib import errors
from plaso.lib import timelib

import pytz  # pylint: disable=wrong-import-order


class EventFiltersArgumentsHelper(interface.ArgumentsHelper):
  """Event filters CLI arguments helper."""

  NAME = 'event_filters'
  DESCRIPTION = 'Event filters command line arguments.'

  _DOCUMENTATION_URL = (
      'https://github.com/log2timeline/plaso/wiki/Event-filters')

  @classmethod
  def AddArguments(cls, argument_group):
    """Adds command line arguments to an argument group.

    This function takes an argument parser or an argument group object and adds
    to it all the command line arguments this helper supports.

    Args:
      argument_group (argparse._ArgumentGroup|argparse.ArgumentParser):
          argparse group.
    """
    argument_group.add_argument(
        '--slice', metavar='DATE', dest='slice', type=str,
        default='', action='store', help=(
            'Create a time slice around a certain date. This parameter, if '
            'defined will display all events that happened X minutes before '
            'and after the defined date. X is controlled by the parameter '
            '--slice_size but defaults to 5 minutes.'))

    argument_group.add_argument(
        '--slice_size', '--slice-size', dest='slice_size', type=int,
        default=5, action='store', help=(
            'Defines the slice size. In the case of a regular time slice it '
            'defines the number of minutes the slice size should be. In the '
            'case of the --slicer it determines the number of events before '
            'and after a filter match has been made that will be included in '
            'the result set. The default value is 5. See --slice or --slicer '
            'for more details about this option.'))

    argument_group.add_argument(
        '--slicer', dest='slicer', action='store_true', default=False, help=(
            'Create a time slice around every filter match. This parameter, '
            'if defined will save all X events before and after a filter '
            'match has been made. X is defined by the --slice_size '
            'parameter.'))

    argument_group.add_argument(
        'filter', nargs='?', action='store', metavar='FILTER', default=None,
        type=str, help=(
            'A filter that can be used to filter the dataset before it '
            'is written into storage. More information about the filters '
            'and how to use them can be found here: {0:s}').format(
                cls._DOCUMENTATION_URL))

  @classmethod
  def ParseOptions(cls, options, configuration_object):
    """Parses and validates options.

    Args:
      options (argparse.Namespace): parser options.
      configuration_object (CLITool): object to be configured by the argument
          helper.

    Raises:
      BadConfigObject: when the configuration object is of the wrong type.
      BadConfigOption: when a configuration parameter fails validation.
    """
    if not isinstance(configuration_object, tools.CLITool):
      raise errors.BadConfigObject(
          'Configuration object is not an instance of CLITool')

    filter_expression = cls._ParseStringOption(options, 'filter')

    event_filter = None
    if filter_expression:
      event_filter = filters_manager.FiltersManager.GetFilterObject(
          filter_expression)
      if not event_filter:
        raise errors.BadConfigOption('Invalid filter expression: {0:s}'.format(
            filter_expression))

    time_slice_event_time_string = getattr(options, 'slice', None)
    time_slice_duration = getattr(options, 'slice_size', 5)
    use_time_slicer = getattr(options, 'slicer', False)

    # The slice and slicer cannot be set at the same time.
    if time_slice_event_time_string and use_time_slicer:
      raise errors.BadConfigOption(
          'Time slice and slicer cannot be used at the same time.')

    time_slice_event_timestamp = None
    if time_slice_event_time_string:
      # Note self._preferred_time_zone is None when not set but represents UTC.
      preferred_time_zone = getattr(
          configuration_object, '_preferred_time_zone', None) or 'UTC'
      timezone = pytz.timezone(preferred_time_zone)
      time_slice_event_timestamp = timelib.Timestamp.FromTimeString(
          time_slice_event_time_string, timezone=timezone)
      if time_slice_event_timestamp is None:
        raise errors.BadConfigOption(
            'Unsupported time slice event date and time: {0:s}'.format(
                time_slice_event_time_string))

    setattr(configuration_object, '_event_filter_expression', filter_expression)

    if event_filter:
      setattr(configuration_object, '_event_filter', event_filter)

    setattr(configuration_object, '_use_time_slicer', use_time_slicer)

    if time_slice_event_timestamp is not None or use_time_slicer:
      # Note that time slicer uses the time slice to determine the duration.
      # TODO: refactor TimeSlice to filters.
      time_slice = time_slices.TimeSlice(
          time_slice_event_timestamp, duration=time_slice_duration)
      setattr(configuration_object, '_time_slice', time_slice)


manager.ArgumentHelperManager.RegisterHelper(EventFiltersArgumentsHelper)



# -*- coding: utf-8 -*-
"""This file contains a plist plugin for the iPod/iPhone storage plist."""

from __future__ import unicode_literals

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import plist
from plaso.parsers.plist_plugins import interface


# TODO: add more attributes.
class IPodPlistEventData(events.EventData):
  """iPod plist event data.

  Attributes:
    device_id (str): unique identifier of the iPod device.
  """

  DATA_TYPE = 'ipod:device:entry'

  def __init__(self):
    """Initializes event data."""
    super(IPodPlistEventData, self).__init__(data_type=self.DATA_TYPE)
    self.device_id = None


class IPodPlugin(interface.PlistPlugin):
  """Plugin to extract iPod/iPad/iPhone device information."""

  NAME = 'ipod_device'
  DESCRIPTION = 'Parser for iPod, iPad and iPhone plist files.'

  PLIST_PATH = 'com.apple.iPod.plist'
  PLIST_KEYS = frozenset(['Devices'])

  # pylint: disable=arguments-differ
  def GetEntries(self, parser_mediator, match=None, **unused_kwargs):
    """Extract device information from the iPod plist.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      match (Optional[dict[str: object]]): keys extracted from PLIST_KEYS.
    """
    devices = match.get('Devices', {})
    for device_identifier, device_information in iter(devices.items()):
      datetime_value = device_information.get('Connected', None)
      if not datetime_value:
        continue

      event_data = IPodPlistEventData()
      event_data.device_id = device_identifier

      # TODO: refactor.
      for key, value in iter(device_information.items()):
        if key == 'Connected':
          continue
        attribute_name = key.lower().replace(' ', '_')
        setattr(event_data, attribute_name, value)

      event = time_events.PythonDatetimeEvent(
          datetime_value, definitions.TIME_DESCRIPTION_LAST_CONNECTED)
      parser_mediator.ProduceEventWithEventData(event, event_data)


plist.PlistParser.RegisterPlugin(IPodPlugin)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the view classes."""

from __future__ import unicode_literals

import sys
import unittest

from plaso.cli import views

from tests import test_lib as shared_test_lib
from tests.cli import test_lib


class TestBaseTableView(views.BaseTableView):
  """Class that implements a table view for testing."""

  def Write(self, unused_output_writer):
    """Writes the table to the output writer.

    Args:
      output_writer: the output writer (instance of OutputWriter).
    """
    return


class BaseTableViewTests(shared_test_lib.BaseTestCase):
  """Tests for the table view interface class."""

  def testAddRow(self):
    """Tests the AddRow function."""
    table_view = TestBaseTableView(
        column_names=['one', 'two', 'three'])

    # Adding rows with the same number of values as columns is permitted.
    table_view.AddRow(['1', '2', '3'])

    # Adding rows with a different number of values as columns is not permitted.
    with self.assertRaises(ValueError):
      table_view.AddRow(['4', '5'])

    table_view.AddRow(['4', '5', '6'])

    table_view = TestBaseTableView()

    table_view.AddRow(['1', '2', '3'])

    # Adding rows with the same number of values is permitted.
    table_view.AddRow(['4', '5', '6'])

    # Adding rows with a different number of values is not permitted.
    with self.assertRaises(ValueError):
      table_view.AddRow(['7', '8'])


class CLITableViewTests(shared_test_lib.BaseTestCase):
  """Tests for the command line table view class."""

  def testWrite(self):
    """Tests the Write function."""
    output_writer = test_lib.TestOutputWriter()

    # Table with columns.
    table_view = views.CLITableView(
        column_names=['Name', 'Description'], title='Title')
    table_view.AddRow(['First name', 'The first name in the table'])
    table_view.AddRow(['Second name', 'The second name in the table'])

    table_view.Write(output_writer)
    string = output_writer.ReadOutput()
    expected_string = (
        b'\n'
        b'************************************ '
        b'Title '
        b'*************************************\n'
        b'       Name : Description\n'
        b'----------------------------------------'
        b'----------------------------------------\n'
        b' First name : The first name in the table\n'
        b'Second name : The second name in the table\n'
        b'----------------------------------------'
        b'----------------------------------------\n')

    # Splitting the string makes it easier to see differences.
    self.assertEqual(string.split(b'\n'), expected_string.split(b'\n'))

    # Table without columns.
    table_view = views.CLITableView(title='Title')
    table_view.AddRow(['Name', 'The name in the table'])
    table_view.AddRow(['Description', 'The description in the table'])

    table_view.Write(output_writer)
    string = output_writer.ReadOutput()
    expected_string = (
        b'\n'
        b'************************************ '
        b'Title '
        b'*************************************\n'
        b'       Name : The name in the table\n'
        b'Description : The description in the table\n'
        b'----------------------------------------'
        b'----------------------------------------\n')

    # Splitting the string makes it easier to see differences.
    self.assertEqual(string.split(b'\n'), expected_string.split(b'\n'))

    # TODO: add test without title.

    # Table with a too large title.
    # TODO: determine if this is the desired behavior.
    title = (
        'In computer programming, a string is traditionally a sequence '
        'of characters, either as a literal constant or as some kind of '
        'variable.')
    table_view = views.CLITableView(
        column_names=['Name', 'Description'], title=title)
    table_view.AddRow(['First name', 'The first name in the table'])
    table_view.AddRow(['Second name', 'The second name in the table'])

    with self.assertRaises(RuntimeError):
      table_view.Write(output_writer)


class CLITabularTableView(shared_test_lib.BaseTestCase):
  """Tests for the command line tabular table view class."""

  def testWrite(self):
    """Tests the Write function."""
    output_writer = test_lib.TestOutputWriter()

    table_view = views.CLITabularTableView(
        column_names=['Name', 'Description'])
    table_view.AddRow(['First name', 'The first name in the table'])
    table_view.AddRow(['Second name', 'The second name in the table'])

    table_view.Write(output_writer)
    string = output_writer.ReadOutput()

    expected_strings = [
        b'Name            Description',
        b'First name      The first name in the table',
        b'Second name     The second name in the table',
        b'']

    if not sys.platform.startswith('win'):
      expected_strings[0] = b'\x1b[1mName            Description\x1b[0m'

    self.assertEqual(string.split(b'\n'), expected_strings)


class MarkdownTableViewTests(shared_test_lib.BaseTestCase):
  """Tests for the Markdown table view class."""

  def testWrite(self):
    """Tests the Write function."""
    output_writer = test_lib.TestOutputWriter()

    # Table with columns.
    table_view = views.MarkdownTableView(
        column_names=['Name', 'Description'], title='Title')
    table_view.AddRow(['First name', 'The first name in the table'])
    table_view.AddRow(['Second name', 'The second name in the table'])

    table_view.Write(output_writer)
    string = output_writer.ReadOutput()
    expected_string = (
        b'### Title\n'
        b'\n'
        b'Name | Description\n'
        b'--- | ---\n'
        b'First name | The first name in the table\n'
        b'Second name | The second name in the table\n'
        b'\n')

    # Splitting the string makes it easier to see differences.
    self.assertEqual(string.split(b'\n'), expected_string.split(b'\n'))

    # Table without columns.
    table_view = views.MarkdownTableView(title='Title')
    table_view.AddRow(['Name', 'The name in the table'])
    table_view.AddRow(['Description', 'The description in the table'])

    table_view.Write(output_writer)
    string = output_writer.ReadOutput()
    expected_string = (
        b'### Title\n'
        b'\n'
        b' | \n'
        b'--- | ---\n'
        b'Name | The name in the table\n'
        b'Description | The description in the table\n'
        b'\n')

    # Splitting the string makes it easier to see differences.
    self.assertEqual(string.split(b'\n'), expected_string.split(b'\n'))

    # TODO: add test without title.


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the Systemd Journal parser."""

from __future__ import unicode_literals

import unittest

try:
  from plaso.parsers import systemd_journal
except ImportError:
  systemd_journal = None

from tests.parsers import test_lib


@unittest.skipIf(systemd_journal is None, 'requires LZMA compression support')
class SystemdJournalParserTest(test_lib.ParserTestCase):
  """Tests for the Systemd Journal parser."""

  def testParse(self):
    """Tests the Parse function."""
    parser = systemd_journal.SystemdJournalParser()
    storage_writer = self._ParseFile([
        'systemd', 'journal', 'system.journal'], parser)

    self.assertEqual(storage_writer.number_of_events, 2101)

    events = list(storage_writer.GetEvents())

    event = events[0]

    self.CheckTimestamp(event.timestamp, '2017-01-27 09:40:55.913258')

    expected_message = (
        'test-VirtualBox [systemd, pid: 1] Started User Manager for '
        'UID 1000.')
    self._TestGetMessageStrings(event, expected_message, expected_message)

    # This event uses XZ compressed data
    event = events[2098]

    self.CheckTimestamp(event.timestamp, '2017-02-06 16:24:32.564585')

    expected_message = 'test-VirtualBox [root, pid: 22921] {0:s}'.format(
        'a' * 692)
    expected_short_message = '{0:s}...'.format(expected_message[:77])
    self._TestGetMessageStrings(event, expected_message, expected_short_message)

  def testParseDirty(self):
    """Tests the Parse function on a 'dirty' journal file."""
    storage_writer = self._CreateStorageWriter()
    parser_mediator = self._CreateParserMediator(storage_writer)
    parser = systemd_journal.SystemdJournalParser()
    path_segments = [
        'systemd', 'journal',
        'system@00053f9c9a4c1e0e-2e18a70e8b327fed.journalTILDE'
    ]
    file_entry = self._GetTestFileEntry(path_segments)
    file_object = file_entry.GetFileObject()

    parser.ParseFileObject(parser_mediator, file_object)

    self.assertEqual(storage_writer.number_of_events, 2211)

    events = list(storage_writer.GetEvents())

    event = events[0]

    self.CheckTimestamp(event.timestamp, '2016-10-24 13:20:01.063423')

    expected_message = (
        'test-VirtualBox [systemd-journald, pid: 569] Runtime journal '
        '(/run/log/journal/) is 1.2M, max 9.9M, 8.6M free.')
    expected_short_message = '{0:s}...'.format(expected_message[:77])
    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    self.assertEqual(storage_writer.number_of_errors, 1)

    errors = list(storage_writer.GetErrors())
    error = errors[0]
    expected_error_message = (
        'Unable to complete parsing journal file: '
        'object offset should be after hash tables (4308912 < 2527472) at '
        'offset 0x0041bfb0')
    self.assertEqual(error.message, expected_error_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The MacOS application usage event formatter."""

from __future__ import unicode_literals

from plaso.formatters import interface
from plaso.formatters import manager


class ApplicationUsageFormatter(interface.EventFormatter):
  """Formatter for a MacOS Application usage event."""

  DATA_TYPE = 'macosx:application_usage'

  FORMAT_STRING = (
      '{application} v.{app_version} (bundle: {bundle_id}). '
      'Launched: {count} time(s)')
  FORMAT_STRING_SHORT = '{application} ({count} time(s))'

  SOURCE_LONG = 'Application Usage'
  SOURCE_SHORT = 'LOG'


manager.FormattersManager.RegisterFormatter(ApplicationUsageFormatter)



# -*- coding: utf-8 -*-
"""Parser for MacOS Keychain files."""

from __future__ import unicode_literals

# INFO: Only supports internet and application passwords,
#       because it is the only data that contains timestamp events.
#       Keychain can also store "secret notes". These notes are stored
#       in the same type than the application format, then, they are already
#       supported. The stored wifi are also application passwords.

# TODO: the AccessControl for each entry has not been implemented. Until now,
#       I know that the AccessControl from Internet and App password are stored
#       using other tables (Symmetric, certificates, etc). Access Control
#       indicates which specific tool, or all, is able to use this entry.


import binascii
import os

import construct

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import errors
from plaso.lib import specification
from plaso.lib import definitions
from plaso.parsers import interface
from plaso.parsers import manager


class KeychainInternetRecordEventData(events.EventData):
  """MacOS keychain internet record event data.

  Attributes:
    account_name (str): name of the account.
    comments (str): comments added by the user.
    entry_name (str): name of the entry.
    protocol (str): internet protocol used, for example "https".
    ssgp_hash (str): hexadecimal values from the password / cert hash.
    text_description (str): description.
    type_protocol (str): sub-protocol used, for example "form".
    where (str): domain name or IP where the password is used.
  """

  DATA_TYPE = 'mac:keychain:internet'

  def __init__(self):
    """Initializes event data."""
    super(KeychainInternetRecordEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.account_name = None
    self.comments = None
    self.entry_name = None
    self.protocol = None
    self.ssgp_hash = None
    self.text_description = None
    self.type_protocol = None
    self.where = None


# TODO: merge with KeychainInternetRecordEventData.
class KeychainApplicationRecordEventData(events.EventData):
  """MacOS keychain application password record event data.

  Attributes:
    account_name (str): name of the account.
    comments (str): comments added by the user.
    entry_name (str): name of the entry.
    ssgp_hash (str): hexadecimal values from the password / cert hash.
    text_description (str): description.
  """

  DATA_TYPE = 'mac:keychain:application'

  def __init__(self):
    """Initializes event data."""
    super(KeychainApplicationRecordEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.account_name = None
    self.comments = None
    self.entry_name = None
    self.ssgp_hash = None
    self.text_description = None


class KeychainParser(interface.FileObjectParser):
  """Parser for Keychain files."""

  NAME = 'mac_keychain'
  DESCRIPTION = 'Parser for MacOS Keychain files.'

  KEYCHAIN_SIGNATURE = b'kych'
  KEYCHAIN_MAJOR_VERSION = 1
  KEYCHAIN_MINOR_VERSION = 0

  RECORD_TYPE_APPLICATION = 0x80000000
  RECORD_TYPE_INTERNET = 0x80000001

  # DB HEADER.
  KEYCHAIN_DB_HEADER = construct.Struct(
      'db_header',
      construct.Bytes('signature', 4),
      construct.UBInt16('major_version'),
      construct.UBInt16('minor_version'),
      construct.UBInt32('header_size'),
      construct.UBInt32('schema_offset'),
      construct.Padding(4))

  # DB SCHEMA.
  KEYCHAIN_DB_SCHEMA = construct.Struct(
      'db_schema',
      construct.UBInt32('size'),
      construct.UBInt32('number_of_tables'))

  # For each number_of_tables, the schema has a TABLE_OFFSET with the
  # offset starting in the DB_SCHEMA.
  TABLE_OFFSET = construct.UBInt32('table_offset')

  TABLE_HEADER = construct.Struct(
      'table_header',
      construct.UBInt32('table_size'),
      construct.UBInt32('record_type'),
      construct.UBInt32('number_of_records'),
      construct.UBInt32('first_record'),
      construct.UBInt32('index_offset'),
      construct.Padding(4),
      construct.UBInt32('recordnumbercount'))

  RECORD_HEADER = construct.Struct(
      'record_header',
      construct.UBInt32('entry_length'),
      construct.Padding(12),
      construct.UBInt32('ssgp_length'),
      construct.Padding(4),
      construct.UBInt32('creation_time'),
      construct.UBInt32('last_modification_time'),
      construct.UBInt32('text_description'),
      construct.Padding(4),
      construct.UBInt32('comments'),
      construct.Padding(8),
      construct.UBInt32('entry_name'),
      construct.Padding(20),
      construct.UBInt32('account_name'),
      construct.Padding(4))

  RECORD_HEADER_APP = construct.Struct(
      'record_entry_app',
      RECORD_HEADER,
      construct.Padding(4))

  RECORD_HEADER_INET = construct.Struct(
      'record_entry_inet',
      RECORD_HEADER,
      construct.UBInt32('where'),
      construct.UBInt32('protocol'),
      construct.UBInt32('type'),
      construct.Padding(4),
      construct.UBInt32('url'))

  TEXT = construct.PascalString(
      'text', length_field=construct.UBInt32('length'))

  TIME = construct.Struct(
      'timestamp',
      construct.String('year', 4),
      construct.String('month', 2),
      construct.String('day', 2),
      construct.String('hour', 2),
      construct.String('minute', 2),
      construct.String('second', 2),
      construct.Padding(2))

  TYPE_TEXT = construct.String('type', 4)

  # TODO: add more protocols.
  _PROTOCOL_TRANSLATION_DICT = {
      'htps': 'https',
      'smtp': 'smtp',
      'imap': 'imap',
      'http': 'http'}

  def _ReadEntryApplication(self, parser_mediator, file_object):
    """Extracts the information from an application password entry.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): a file-like object.
    """
    record_offset = file_object.tell()
    try:
      record_struct = self.RECORD_HEADER_APP.parse_stream(file_object)
    except (IOError, construct.FieldError):
      parser_mediator.ProduceExtractionError(
          'unable to parse record structure at offset: 0x{0:08x}'.format(
              record_offset))
      return

    (ssgp_hash, creation_time, last_modification_time, text_description,
     comments, entry_name, account_name) = self._ReadEntryHeader(
         parser_mediator, file_object, record_struct.record_header,
         record_offset)

    # Move to the end of the record.
    next_record_offset = (
        record_offset + record_struct.record_header.entry_length)
    file_object.seek(next_record_offset, os.SEEK_SET)

    event_data = KeychainApplicationRecordEventData()
    event_data.account_name = account_name
    event_data.comments = comments
    event_data.entry_name = entry_name
    event_data.ssgp_hash = ssgp_hash
    event_data.text_description = text_description

    if creation_time:
      event = time_events.DateTimeValuesEvent(
          creation_time, definitions.TIME_DESCRIPTION_CREATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)

    if last_modification_time:
      event = time_events.DateTimeValuesEvent(
          last_modification_time, definitions.TIME_DESCRIPTION_MODIFICATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)

  def _ReadEntryHeader(
      self, parser_mediator, file_object, record, record_offset):
    """Read the common record attributes.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_entry (dfvfs.FileEntry): a file entry object.
      file_object (dfvfs.FileIO): a file-like object.
      record (construct.Struct): record header structure.
      record_offset (int): offset of the start of the record.

    Returns:
      A tuple containing:
        ssgp_hash: Hash of the encrypted data (passwd, cert, note).
        creation_time (dfdatetime.TimeElements): entry creation time or None.
        last_modification_time ((dfdatetime.TimeElements): entry last
            modification time or None.
        text_description: A brief description of the entry.
        entry_name: Name of the entry
        account_name: Name of the account.
    """
    # TODO: reduce number of seeks and/or offset calculations needed
    # for parsing.

    # Info: The hash header always start with the string ssgp follow by
    #       the hash. Furthermore The fields are always a multiple of four.
    #       Then if it is not multiple the value is padded by 0x00.
    ssgp_hash = binascii.hexlify(file_object.read(record.ssgp_length)[4:])

    creation_time = None

    structure_offset = record_offset + record.creation_time - 1
    file_object.seek(structure_offset, os.SEEK_SET)

    try:
      time_structure = self.TIME.parse_stream(file_object)
    except construct.FieldError as exception:
      time_structure = None
      parser_mediator.ProduceExtractionError(
          'unable to parse creation time with error: {0!s}'.format(exception))

    if time_structure:
      time_elements_tuple = (
          time_structure.year, time_structure.month, time_structure.day,
          time_structure.hour, time_structure.minute, time_structure.second)

      creation_time = dfdatetime_time_elements.TimeElements()
      try:
        creation_time.CopyFromStringTuple(
            time_elements_tuple=time_elements_tuple)
      except ValueError:
        creation_time = None
        parser_mediator.ProduceExtractionError(
            'invalid creation time value: {0!s}'.format(time_elements_tuple))

    last_modification_time = None

    structure_offset = record_offset + record.last_modification_time - 1
    file_object.seek(structure_offset, os.SEEK_SET)

    try:
      time_structure = self.TIME.parse_stream(file_object)
    except construct.FieldError as exception:
      time_structure = None
      parser_mediator.ProduceExtractionError(
          'unable to parse last modification time with error: {0!s}'.format(
              exception))

    if time_structure:
      time_elements_tuple = (
          time_structure.year, time_structure.month, time_structure.day,
          time_structure.hour, time_structure.minute, time_structure.second)

      last_modification_time = dfdatetime_time_elements.TimeElements()
      try:
        last_modification_time.CopyFromStringTuple(
            time_elements_tuple=time_elements_tuple)
      except ValueError:
        last_modification_time = None
        parser_mediator.ProduceExtractionError(
            'invalid last modification time value: {0!s}'.format(
                time_elements_tuple))

    text_description = 'N/A'
    if record.text_description:
      structure_offset = record_offset + record.text_description - 1
      file_object.seek(structure_offset, os.SEEK_SET)

      try:
        text_description = self.TEXT.parse_stream(file_object)
      except construct.FieldError as exception:
        parser_mediator.ProduceExtractionError(
            'unable to parse text description with error: {0!s}'.format(
                exception))

    comments = 'N/A'
    if record.comments:
      structure_offset = record_offset + record.comments - 1
      file_object.seek(structure_offset, os.SEEK_SET)

      try:
        comments = self.TEXT.parse_stream(file_object)
      except construct.FieldError as exception:
        parser_mediator.ProduceExtractionError(
            'unable to parse comments with error: {0!s}'.format(exception))

    structure_offset = record_offset + record.entry_name - 1
    file_object.seek(structure_offset, os.SEEK_SET)

    try:
      entry_name = self.TEXT.parse_stream(file_object)
    except construct.FieldError as exception:
      entry_name = 'N/A'
      parser_mediator.ProduceExtractionError(
          'unable to parse entry name with error: {0!s}'.format(exception))

    structure_offset = record_offset + record.account_name - 1
    file_object.seek(structure_offset, os.SEEK_SET)

    try:
      account_name = self.TEXT.parse_stream(file_object)
    except construct.FieldError as exception:
      account_name = 'N/A'
      parser_mediator.ProduceExtractionError(
          'unable to parse account name with error: {0!s}'.format(exception))

    return (
        ssgp_hash, creation_time, last_modification_time,
        text_description, comments, entry_name, account_name)

  def _ReadEntryInternet(self, parser_mediator, file_object):
    """Extracts the information from an Internet password entry.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): a file-like object.
    """
    record_offset = file_object.tell()
    try:
      record_header_struct = self.RECORD_HEADER_INET.parse_stream(file_object)
    except (IOError, construct.FieldError):
      parser_mediator.ProduceExtractionError((
          'unable to parse record header structure at offset: '
          '0x{0:08x}').format(record_offset))
      return

    (ssgp_hash, creation_time, last_modification_time, text_description,
     comments, entry_name, account_name) = self._ReadEntryHeader(
         parser_mediator, file_object, record_header_struct.record_header,
         record_offset)

    if not record_header_struct.where:
      where = 'N/A'
      protocol = 'N/A'
      type_protocol = 'N/A'

    else:
      offset = record_offset + record_header_struct.where - 1
      file_object.seek(offset, os.SEEK_SET)
      where = self.TEXT.parse_stream(file_object)

      offset = record_offset + record_header_struct.protocol - 1
      file_object.seek(offset, os.SEEK_SET)
      protocol = self.TYPE_TEXT.parse_stream(file_object)

      offset = record_offset + record_header_struct.type - 1
      file_object.seek(offset, os.SEEK_SET)
      type_protocol = self.TEXT.parse_stream(file_object)
      type_protocol = self._PROTOCOL_TRANSLATION_DICT.get(
          type_protocol, type_protocol)

      if record_header_struct.url:
        offset = record_offset + record_header_struct.url - 1
        file_object.seek(offset, os.SEEK_SET)
        url = self.TEXT.parse_stream(file_object)
        where = '{0:s}{1:s}'.format(where, url)

    # Move to the end of the record.
    next_record_offset = (
        record_offset + record_header_struct.record_header.entry_length)
    file_object.seek(next_record_offset, os.SEEK_SET)

    event_data = KeychainInternetRecordEventData()
    event_data.account_name = account_name
    event_data.comments = comments
    event_data.entry_name = entry_name
    event_data.protocol = protocol
    event_data.ssgp_hash = ssgp_hash
    event_data.text_description = text_description
    event_data.type_protocol = type_protocol
    event_data.where = where

    if creation_time:
      event = time_events.DateTimeValuesEvent(
          creation_time, definitions.TIME_DESCRIPTION_CREATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)

    if last_modification_time:
      event = time_events.DateTimeValuesEvent(
          last_modification_time, definitions.TIME_DESCRIPTION_MODIFICATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)

  def _ReadTableOffsets(self, parser_mediator, file_object):
    """Reads the table offsets.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): a file-like object.

    Returns:
      list[int]: table offsets.
    """
    # INFO: The HEADER KEYCHAIN:
    # [DBHEADER] + [DBSCHEMA] + [OFFSET TABLE A] + ... + [OFFSET TABLE Z]
    # Where the table offset is relative to the first byte of the DB Schema,
    # then we must add to this offset the size of the [DBHEADER].
    # Read the database schema and extract the offset for all the tables.
    # They are ordered by file position from the top to the bottom of the file.
    table_offsets = []

    try:
      db_schema_struct = self.KEYCHAIN_DB_SCHEMA.parse_stream(file_object)
    except (IOError, construct.FieldError):
      parser_mediator.ProduceExtractionError(
          'unable to parse database schema structure')
      return []

    for index in range(db_schema_struct.number_of_tables):
      try:
        table_offset = self.TABLE_OFFSET.parse_stream(file_object)
      except (IOError, construct.FieldError):
        parser_mediator.ProduceExtractionError(
            'unable to parse table offsets: {0:d}'.format(index))
        return []

      table_offsets.append(table_offset + self.KEYCHAIN_DB_HEADER.sizeof())

    return table_offsets

  @classmethod
  def GetFormatSpecification(cls):
    """Retrieves the format specification.

    Returns:
      FormatSpecification: format specification.
    """
    format_specification = specification.FormatSpecification(cls.NAME)
    format_specification.AddNewSignature(
        cls.KEYCHAIN_SIGNATURE, offset=0)
    return format_specification

  def ParseFileObject(self, parser_mediator, file_object, **kwargs):
    """Parses a MacOS keychain file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): a file-like object.

    Raises:
      UnableToParseFile: when the file cannot be parsed.
    """
    try:
      db_header = self.KEYCHAIN_DB_HEADER.parse_stream(file_object)
    except (IOError, construct.FieldError):
      raise errors.UnableToParseFile('Unable to parse file header.')

    if db_header.signature != self.KEYCHAIN_SIGNATURE:
      raise errors.UnableToParseFile('Not a MacOS keychain file.')

    if (db_header.major_version != self.KEYCHAIN_MAJOR_VERSION or
        db_header.minor_version != self.KEYCHAIN_MINOR_VERSION):
      parser_mediator.ProduceExtractionError(
          'unsupported format version: {0:s}.{1:s}'.format(
              db_header.major_version, db_header.minor_version))
      return

    # TODO: document format and determine if -1 offset correction is needed.
    table_offsets = self._ReadTableOffsets(parser_mediator, file_object)
    for table_offset in table_offsets:
      # Skipping X bytes, unknown data at this point.
      file_object.seek(table_offset, os.SEEK_SET)

      try:
        table = self.TABLE_HEADER.parse_stream(file_object)
      except (IOError, construct.FieldError):
        parser_mediator.ProduceExtractionError(
            'unable to parse table structure at offset: 0x{0:08x}'.format(
                table_offset))
        continue

      # Table_offset: absolute byte in the file where the table starts.
      # table.first_record: first record in the table, relative to the
      #                     first byte of the table.
      file_object.seek(table_offset + table.first_record, os.SEEK_SET)

      if table.record_type == self.RECORD_TYPE_INTERNET:
        for _ in range(table.number_of_records):
          self._ReadEntryInternet(parser_mediator, file_object)

      elif table.record_type == self.RECORD_TYPE_APPLICATION:
        for _ in range(table.number_of_records):
          self._ReadEntryApplication(parser_mediator, file_object)


manager.ParsersManager.RegisterParser(KeychainParser)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the shell item event attribute container."""

from __future__ import unicode_literals

import unittest

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import time_events

from tests import test_lib as shared_test_lib


class TimestampEventTest(shared_test_lib.BaseTestCase):
  """Tests for the Plaso timestamp-based event attribute container."""

  def testGetAttributeNames(self):
    """Tests the GetAttributeNames function."""
    attribute_container = time_events.TimestampEvent(0, 'usage')

    expected_attribute_names = [
        'data_type', 'display_name', 'filename', 'hostname', 'inode',
        'offset', 'pathspec', 'tag', 'timestamp', 'timestamp_desc']

    attribute_names = sorted(attribute_container.GetAttributeNames())

    self.assertEqual(attribute_names, expected_attribute_names)


class DateTimeValuesEventTest(shared_test_lib.BaseTestCase):
  """Tests for the dfDateTime-based event attribute container."""

  def testGetAttributeNames(self):
    """Tests the GetAttributeNames function."""
    posix_time = dfdatetime_posix_time.PosixTime(timestamp=0)
    attribute_container = time_events.DateTimeValuesEvent(posix_time, 'usage')

    expected_attribute_names = [
        'data_type', 'display_name', 'filename', 'hostname', 'inode',
        'offset', 'pathspec', 'tag', 'timestamp', 'timestamp_desc']

    attribute_names = sorted(attribute_container.GetAttributeNames())

    self.assertEqual(attribute_names, expected_attribute_names)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""An LL(1) lexer. This lexer is very tolerant of errors and can resync.

This lexer is originally copied from the GRR project:
https://code.google.com/p/grr
"""

from __future__ import unicode_literals

import logging
import re

from plaso.lib import errors


class Token(object):
  """A token action."""

  def __init__(self, state_regex, regex, actions, next_state, flags=re.I):
    """Initializes the token object.

    Args:

      state_regex: If this regular expression matches the current state this
                   rule is considered.
      regex: A regular expression to try and match from the current point.
      actions: A command separated list of method names in the Lexer to call.
      next_state: The next state we transition to if this Token matches.
      flags: re flags.
    """
    self.state_regex = re.compile(
        state_regex, re.DOTALL | re.M | re.S | re.U | flags)
    self.regex = re.compile(regex, re.DOTALL | re.M | re.S | re.U | flags)
    self.re_str = regex
    self.actions = []
    if actions:
      self.actions = actions.split(',')

    self.next_state = next_state


class Lexer(object):
  """A generic feed lexer."""
  _CONTINUE_STATE = 'CONTINUE'
  _INITIAL_STATE = 'INITIAL'

  _ERROR_TOKEN = 'Error'

  # A list of Token() instances.
  tokens = []

  def __init__(self, data=''):
    """Initializes the lexer object.

    Args:
      data: optional initial data to be processed by the lexer.
    """
    super(Lexer, self).__init__()
    self.buffer = data
    self.error = 0
    self.flags = 0
    self.processed = 0
    self.processed_buffer = ''
    self.state = self._INITIAL_STATE
    self.state_stack = []
    self.verbose = 0

  def NextToken(self):
    """Fetch the next token by trying to match any of the regexes in order."""
    current_state = self.state
    for token in self.tokens:
      # Does the rule apply to us?
      if not token.state_regex.match(current_state):
        continue

      # Try to match the rule
      m = token.regex.match(self.buffer)
      if not m:
        continue

      # The match consumes the data off the buffer (the handler can put it back
      # if it likes)
      # TODO: using joins might be more efficient here.
      self.processed_buffer += self.buffer[:m.end()]
      self.buffer = self.buffer[m.end():]
      self.processed += m.end()

      next_state = token.next_state
      for action in token.actions:

        # Is there a callback to handle this action?
        callback = getattr(self, action, self.Default)

        # Allow a callback to skip other callbacks.
        try:
          possible_next_state = callback(string=m.group(0), match=m)
          if possible_next_state == self._CONTINUE_STATE:
            continue
          # Override the state from the Token
          elif possible_next_state:
            next_state = possible_next_state
        except errors.ParseError as exception:
          self.Error(exception)

      # Update the next state
      if next_state:
        self.state = next_state

      return token

    # Check that we are making progress - if we are too full, we assume we are
    # stuck.
    self.Error('Expected {0:s}'.format(self.state))
    self.processed_buffer += self.buffer[:1]
    self.buffer = self.buffer[1:]
    return self._ERROR_TOKEN

  def Close(self):
    """A convenience function to force us to parse all the data."""
    while self.NextToken():
      if not self.buffer:
        return

  def Default(self, **kwarg):
    """The default callback handler."""
    logging.debug('Default handler: {0:s}'.format(kwarg))

  def Empty(self):
    """Returns a boolean indicating if the buffer is empty."""
    return not self.buffer

  def Error(self, message=None, weight=1):
    """Log an error down.

    Args:
      message: optional error message.
      weight: optional error weight.
    """
    logging.debug('Error({0:d}): {1:s}'.format(weight, message))
    # Keep a count of errors
    self.error += weight

  def PushState(self, **unused_kwargs):
    """Push the current state on the state stack."""
    logging.debug('Storing state {0:s}'.format(repr(self.state)))
    self.state_stack.append(self.state)

  def PopState(self, **unused_kwargs):
    """Pop the previous state from the stack."""
    try:
      self.state = self.state_stack.pop()
      logging.debug('Returned state to {0:s}'.format(self.state))

      return self.state
    except IndexError:
      self.Error(
          'Tried to pop the state but failed - possible recursion error')

  def Feed(self, data):
    """Feed the buffer with data.

    Args:
      data: data to be processed by the lexer.
    """
    self.buffer = ''.join([self.buffer, data])

  def PushBack(self, string='', **unused_kwargs):
    """Push the match back on the stream.

    Args:
      string: optional data.
    """
    self.buffer = string + self.buffer
    self.processed_buffer = self.processed_buffer[:-len(string)]


class SelfFeederMixIn(Lexer):
  """This mixin is used to make a lexer which feeds itself.

  Note that self.file_object must be the file object we read from.
  """

  # TODO: fix this, file object either needs to be set or not passed here.
  def __init__(self, file_object=None):
    """Initializes the lexer feeder min object.

    Args:
      file_object: Optional file-like object.
    """
    super(SelfFeederMixIn, self).__init__()
    self.file_object = file_object

  # pylint: disable=arguments-differ
  def Feed(self, size=512):
    """Feed data into the buffer.

    Args:
      size: optional data size to read form the file-like object.
    """
    data = self.file_object.read(size)
    Lexer.Feed(self, data)
    return len(data)

  def NextToken(self):
    """Retrieves the next token.

    Returns:
      The next token (instance of Token) or None.
    """
    # If we don't have enough data - feed ourselves: We assume
    # that we must have at least one sector in our buffer.
    if len(self.buffer) < 512:
      if self.Feed() == 0 and not self.buffer:
        return None

    return Lexer.NextToken(self)


class Expression(object):
  """A class representing an expression."""
  attribute = None
  args = None
  operator = None

  # The expected number of args
  number_of_args = 1

  def __init__(self):
    """Initializes the expression object."""
    self.args = []

  def __str__(self):
    """Return a string representation of the expression."""
    return 'Expression: ({0:s}) ({1:s}) {2:s}'.format(
        self.attribute, self.operator, self.args)

  def AddArg(self, arg):
    """Adds a new arg to this expression.

    Args:
       arg: The argument to add (string).

    Returns:
      True if this arg is the last arg, False otherwise.

    Raises:
      ParseError: If there are too many args.
    """
    self.args.append(arg)
    if len(self.args) > self.number_of_args:
      raise errors.ParseError('Too many args for this expression.')

    elif len(self.args) == self.number_of_args:
      return True

    return False

  def Compile(self, unused_filter_implementation):
    """Given a filter implementation, compile this expression."""
    raise NotImplementedError(
        '{0:s} does not implement Compile.'.format(self.__class__.__name__))

  # TODO: rename this function to GetTreeAsString or equivalent.
  def PrintTree(self, depth=''):
    """Print the tree."""
    return '{0:s} {1:s}'.format(depth, self)

  def SetAttribute(self, attribute):
    """Set the attribute."""
    self.attribute = attribute

  def SetOperator(self, operator):
    """Set the operator."""
    self.operator = operator


class BinaryExpression(Expression):
  """An expression which takes two other expressions."""

  def __init__(self, operator='', part=None):
    """Initializes the expression object."""
    self.operator = operator
    self.args = []
    if part:
      self.args.append(part)
    super(BinaryExpression, self).__init__()

  def __str__(self):
    """Return a string representation of the binary expression."""
    return 'Binary Expression: {0:s} {1:s}'.format(
        self.operator, [str(x) for x in self.args])

  def AddOperands(self, lhs, rhs):
    """Add an operand."""
    if isinstance(lhs, Expression) and isinstance(rhs, Expression):
      self.args = [lhs, rhs]
    else:
      raise errors.ParseError(
          'Expected expression, got {0:s} {1:s} {2:s}'.format(
              lhs, self.operator, rhs))

  # TODO: rename this function to GetTreeAsString or equivalent.
  def PrintTree(self, depth=''):
    """Print the tree."""
    result = '{0:s}{1:s}\n'.format(depth, self.operator)
    for part in self.args:
      result += '{0:s}-{1:s}\n'.format(depth, part.PrintTree(depth + '  '))

    return result

  def Compile(self, filter_implementation):
    """Compile the binary expression into a filter object."""
    operator = self.operator.lower()
    if operator == 'and' or operator == '&&':
      method = 'AndFilter'
    elif operator == 'or' or operator == '||':
      method = 'OrFilter'
    else:
      raise errors.ParseError(
          'Invalid binary operator {0:s}'.format(operator))

    args = [x.Compile(filter_implementation) for x in self.args]
    return getattr(filter_implementation, method)(*args)


class IdentityExpression(Expression):
  """An Expression which always evaluates to True."""

  def Compile(self, filter_implementation):
    """Compile the expression."""
    return filter_implementation.IdentityFilter()


class SearchParser(Lexer):
  """This parser can parse the mini query language and build an AST.

  Examples of valid syntax:
    filename contains "foo" and (size > 100k or date before "2011-10")
    date between 2011 and 2010
    files older than 1 year
  """

  expression_cls = Expression
  binary_expression_cls = BinaryExpression

  tokens = [
      # Double quoted string
      Token('STRING', '"', 'PopState,StringFinish', None),
      Token('STRING', r'\\(.)', 'StringEscape', None),
      Token('STRING', r'[^\\"]+', 'StringInsert', None),

      # Single quoted string
      Token('SQ_STRING', '\'', 'PopState,StringFinish', None),
      Token('SQ_STRING', r'\\(.)', 'StringEscape', None),
      Token('SQ_STRING', r'[^\\\']+', 'StringInsert', None),

      # TODO: Implement a unary not operator.
      # The first thing we see in the initial state takes up to the ATTRIBUTE
      Token('INITIAL', r'(and|or|\&\&|\|\|)', 'BinaryOperator', None),
      Token('INITIAL', r'[^\s\(\)]', 'PushState,PushBack', 'ATTRIBUTE'),
      Token('INITIAL', r'\(', 'BracketOpen', None),
      Token('INITIAL', r'\)', 'BracketClose', None),

      Token('ATTRIBUTE', r'[\w._0-9]+', 'StoreAttribute', 'OPERATOR'),
      Token('OPERATOR', r'[a-z0-9<>=\-\+\!\^\&%]+', 'StoreOperator',
            'ARG_LIST'),
      Token('OPERATOR', r'(!=|[<>=])', 'StoreSpecialOperator', 'ARG_LIST'),
      Token('ARG_LIST', r'[^\s\'"]+', 'InsertArg', None),

      # Start a string.
      Token('.', '"', 'PushState,StringStart', 'STRING'),
      Token('.', '\'', 'PushState,StringStart', 'SQ_STRING'),

      # Skip whitespace.
      Token('.', r'\s+', None, None),
      ]

  def __init__(self, data):
    """Initializes the search parser object."""
    # Holds expression
    self.current_expression = self.expression_cls()
    self.filter_string = data
    # The token stack
    self.stack = []
    self.string = None

    Lexer.__init__(self, data)

  def BinaryOperator(self, string=None, **unused_kwargs):
    """Set the binary operator."""
    self.stack.append(self.binary_expression_cls(string))

  def BracketOpen(self, **unused_kwargs):
    """Define an open bracket."""
    self.stack.append('(')

  def BracketClose(self, **unused_kwargs):
    """Close the bracket."""
    self.stack.append(')')

  def StringStart(self, **unused_kwargs):
    """Initialize the string."""
    self.string = ''

  def StringEscape(self, string, match, **unused_kwargs):
    """Escape backslashes found inside a string quote.

    Backslashes followed by anything other than ['"rnbt] will just be included
    in the string.

    Args:
      string: The string that matched.
      match: the match object (instance of re.MatchObject).
             Where match.group(1) contains the escaped code.
    """
    if match.group(1) in '\'"rnbt':
      self.string += string.decode('string_escape')
    else:
      self.string += string

  def StringInsert(self, string='', **unused_kwargs):
    """Add to the string."""
    self.string += string

  def StringFinish(self, **unused_kwargs):
    """Finish the string operation."""
    if self.state == 'ATTRIBUTE':
      return self.StoreAttribute(string=self.string)

    elif self.state == 'ARG_LIST':
      return self.InsertArg(string=self.string)
    return None

  def StoreAttribute(self, string='', **unused_kwargs):
    """Store the attribute."""
    logging.debug('Storing attribute {0:s}'.format(repr(string)))

    # TODO: Update the expected number_of_args
    try:
      self.current_expression.SetAttribute(string)
    except AttributeError:
      raise errors.ParseError('Invalid attribute \'{0:s}\''.format(string))

    return 'OPERATOR'

  def StoreOperator(self, string='', **unused_kwargs):
    """Store the operator."""
    logging.debug('Storing operator {0:s}'.format(repr(string)))
    self.current_expression.SetOperator(string)

  def InsertArg(self, string='', **unused_kwargs):
    """Insert an arg to the current expression."""
    logging.debug('Storing Argument {0:s}'.format(string))

    # This expression is complete
    if self.current_expression.AddArg(string):
      self.stack.append(self.current_expression)
      self.current_expression = self.expression_cls()
      return self.PopState()
    return None

  def _CombineBinaryExpressions(self, operator):
    """Combine binary expressions."""
    for i in range(1, len(self.stack)-1):
      item = self.stack[i]
      if (isinstance(item, BinaryExpression) and item.operator == operator and
          isinstance(self.stack[i-1], Expression) and
          isinstance(self.stack[i+1], Expression)):
        lhs = self.stack[i-1]
        rhs = self.stack[i+1]

        self.stack[i].AddOperands(lhs, rhs)
        self.stack[i-1] = None
        self.stack[i+1] = None

    self.stack = filter(None, self.stack)

  def _CombineParenthesis(self):
    """Combine parenthesis."""
    for i in range(len(self.stack)-2):
      if (self.stack[i] == '(' and self.stack[i+2] == ')' and
          isinstance(self.stack[i+1], Expression)):
        self.stack[i] = None
        self.stack[i+2] = None

    self.stack = filter(None, self.stack)

  def Reduce(self):
    """Reduce the token stack into an AST."""
    # Check for sanity
    if self.state != 'INITIAL':
      self.Error('Premature end of expression')

    length = len(self.stack)
    while length > 1:
      # Precedence order
      self._CombineParenthesis()
      self._CombineBinaryExpressions('and')
      self._CombineBinaryExpressions('or')

      # No change
      if len(self.stack) == length:
        break
      length = len(self.stack)

    if length != 1:
      self.Error('Illegal query expression')

    return self.stack[0]

  def Error(self, message=None, unused_weight=1):
    """Raise an error message."""
    raise errors.ParseError(
        '{0:s} in position {1:s}: {2:s} <----> {3:s} )'.format(
            message, len(self.processed_buffer), self.processed_buffer,
            self.buffer))

  def Parse(self):
    """Parse."""
    if not self.filter_string:
      return IdentityExpression()

    self.Close()
    return self.Reduce()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the event tag index."""

from __future__ import unicode_literals

import os
import unittest

from plaso.storage import event_tag_index
from plaso.storage import identifiers
from plaso.storage.sqlite import sqlite_file

from tests import test_lib as shared_test_lib
from tests.storage import test_lib


class EventTagIndexTest(test_lib.StorageTestCase):
  """Tests for the event tag index."""

  # pylint: disable=protected-access

  def _CreateTestStorageFileWithTags(self, path):
    """Creates a storage file with event tags for testing.

    Args:
      path (str): path of the storage file.

    Returns:
      StorageFile: storage file.
    """
    storage_file = sqlite_file.SQLiteStorageFile()
    storage_file.Open(path=path, read_only=False)

    test_events = self._CreateTestEvents()
    for event in test_events:
      storage_file.AddEvent(event)

    test_event_tags = self._CreateTestEventTags(test_events)
    storage_file.AddEventTags(test_event_tags[:-1])
    storage_file.AddEventTags(test_event_tags[-1:])

    storage_file.Close()

  @shared_test_lib.skipUnlessHasTestFile(['psort_test.plaso'])
  def testBuild(self):
    """Tests the _Build function."""
    test_index = event_tag_index.EventTagIndex()

    self.assertIsNone(test_index._index)

    test_file = self._GetTestFilePath(['psort_test.plaso'])
    storage_file = sqlite_file.SQLiteStorageFile()
    storage_file.Open(path=test_file)
    test_index._Build(storage_file)
    storage_file.Close()

    self.assertIsNotNone(test_index._index)

  def testGetEventTagByIdentifier(self):
    """Tests the GetEventTagByIdentifier function."""
    test_index = event_tag_index.EventTagIndex()

    with shared_test_lib.TempDirectory() as temp_directory:
      temp_file = os.path.join(temp_directory, 'storage.plaso')
      self._CreateTestStorageFileWithTags(temp_file)

      storage_file = sqlite_file.SQLiteStorageFile()
      storage_file.Open(path=temp_file)

      event_identifier = identifiers.SQLTableIdentifier('event', 1)
      event_tag = test_index.GetEventTagByIdentifier(
          storage_file, event_identifier)
      self.assertIsNotNone(event_tag)
      self.assertEqual(event_tag.comment, 'My comment')

      event_identifier = identifiers.SQLTableIdentifier('event', 99)
      event_tag = test_index.GetEventTagByIdentifier(
          storage_file, event_identifier)
      self.assertIsNone(event_tag)

      storage_file.Close()

  # TODO: add test for SetEventTag.


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Function decorators."""

from __future__ import unicode_literals

import warnings


# pylint: disable=invalid-name

def deprecated(function):
  """Decorator to mark functions or methods as deprecated."""

  def IssueDeprecationWarning(*args, **kwargs):
    """Issue a deprecation warning."""
    warnings.simplefilter('default', DeprecationWarning)
    warnings.warn('Call to deprecated function: {0:s}.'.format(
        function.__name__), category=DeprecationWarning, stacklevel=2)

    return function(*args, **kwargs)

  IssueDeprecationWarning.__name__ = function.__name__
  IssueDeprecationWarning.__doc__ = function.__doc__
  IssueDeprecationWarning.__dict__.update(function.__dict__)
  return IssueDeprecationWarning



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the Docker JSON event formatters."""

from __future__ import unicode_literals

import unittest

from plaso.formatters import docker

from tests.formatters import test_lib


class DockerJSONFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the Docker JSON event formatters."""

  def testInitializations(self):
    """Tests the initialization of the formatters."""
    event_formatter = docker.DockerBaseEventFormatter()
    self.assertIsNotNone(event_formatter)

    event_formatter1 = docker.DockerContainerEventFormatter()
    self.assertIsNotNone(event_formatter1)

    event_formatter2 = docker.DockerContainerLogEventFormatter()
    self.assertIsNotNone(event_formatter2)

    event_formatter3 = docker.DockerLayerEventFormatter()
    self.assertIsNotNone(event_formatter3)


  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""

    event_formatter = docker.DockerBaseEventFormatter()

    expected_attribute_names = []

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

    event_formatter1 = docker.DockerContainerEventFormatter()

    expected_attribute_names = ['action',
                                'container_id',
                                'container_name']

    self._TestGetFormatStringAttributeNames(
        event_formatter1, expected_attribute_names)

    event_formatter2 = docker.DockerContainerLogEventFormatter()

    expected_attribute_names = ['container_id',
                                'log_line',
                                'log_source']

    self._TestGetFormatStringAttributeNames(
        event_formatter2, expected_attribute_names)

    event_formatter3 = docker.DockerLayerEventFormatter()

    expected_attribute_names = ['command',
                                'layer_id']

    self._TestGetFormatStringAttributeNames(
        event_formatter3, expected_attribute_names)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The format specification classes."""

from __future__ import unicode_literals


class Signature(object):
  """The format specification signature.

  The signature consists of a byte string pattern, an optional
  offset relative to the start of the data, and a value to indicate
  if the pattern is bound to the offset.
  """
  def __init__(self, pattern, offset=None):
    """Initializes a format specification signature.

    The signatures can be defined in 3 different "modes":
    * where offset >= 0, which represents that the signature is bound to the
      start of the data and only the relevant part is scanned;
    * where offset < 0, which represents that the signature is bound to the
      end of the data and only the relevant part is scanned;
    * offset == None, which represents that the signature is not offset bound
      and that all of the data is scanned.

    Args:
      pattern (bytes): pattern of the signature. Wildcards or regular
          expressions (regexp) are not supported.
      offset (int): offset of the signature. None is used to indicate
          the signature has no offset. A positive offset is relative from
          the start of the data a negative offset is relative from the end
          of the data.
    """
    super(Signature, self).__init__()
    self.identifier = None
    self.offset = offset
    self.pattern = pattern

  def SetIdentifier(self, identifier):
    """Sets the identifier of the signature in the specification store.

    Args:
      identifier (str): unique signature identifier for a specification store.
    """
    self.identifier = identifier


class FormatSpecification(object):
  """The format specification."""

  def __init__(self, identifier, text_format=False):
    """Initializes a format specification.

    Args:
      identifier (str): unique name for the format.
      text_format (Optional[bool]): True if the format is a text format,
          False otherwise.
    """
    super(FormatSpecification, self).__init__()
    self._text_format = text_format
    self.identifier = identifier
    self.signatures = []

  def AddNewSignature(self, pattern, offset=None):
    """Adds a signature.

    Args:
      pattern (bytes): pattern of the signature.
      offset (int): offset of the signature. None is used to indicate
          the signature has no offset. A positive offset is relative from
          the start of the data a negative offset is relative from the end
          of the data.
    """
    self.signatures.append(Signature(pattern, offset=offset))

  def IsTextFormat(self):
    """Determines if the format is a text format.

    Returns:
      bool: True if the format is a text format, False otherwise.
    """
    return self._text_format


class FormatSpecificationStore(object):
  """The store for format specifications."""

  def __init__(self):
    """Initializes a specification store."""
    super(FormatSpecificationStore, self).__init__()
    self._format_specifications = {}
    # Maps signature identifiers to format specifications.
    self._signature_map = {}

  @property
  def specifications(self):
    """iterator: specifications iterator."""
    return iter(self._format_specifications.values())

  def AddNewSpecification(self, identifier):
    """Adds a new format specification.

    Args:
      identifier (str): format identifier, which should be unique for the store.

    Returns:
      FormatSpecification: format specification.

    Raises:
      KeyError: if the store already contains a specification with
                the same identifier.
    """
    if identifier in self._format_specifications:
      raise KeyError(
          'Format specification {0:s} is already defined in store.'.format(
              identifier))

    self._format_specifications[identifier] = FormatSpecification(identifier)

    return self._format_specifications[identifier]

  def AddSpecification(self, specification):
    """Adds a format specification.

    Args:
      specification (FormatSpecification): format specification.

    Raises:
      KeyError: if the store already contains a specification with
                the same identifier.
    """
    if specification.identifier in self._format_specifications:
      raise KeyError(
          'Format specification {0:s} is already defined in store.'.format(
              specification.identifier))

    self._format_specifications[specification.identifier] = specification

    for signature in specification.signatures:
      signature_index = len(self._signature_map)

      signature_identifier = '{0:s}:{1:d}'.format(
          specification.identifier, signature_index)

      if signature_identifier in self._signature_map:
        raise KeyError('Signature {0:s} is already defined in map.'.format(
            signature_identifier))

      signature.SetIdentifier(signature_identifier)
      self._signature_map[signature_identifier] = specification

  def GetSpecificationBySignature(self, signature_identifier):
    """Retrieves a specification mapped to a signature identifier.

    Args:
      identifier (str): unique signature identifier for a specification store.

    Returns:
      FormatSpecification: format specification or None if the signature
          identifier does not exist within the specification store.
    """
    return self._signature_map.get(signature_identifier, None)



# -*- coding: utf-8 -*-
"""An output module that writes event with geography data to a KML XML file.

The Keyhole Markup Language (KML) is an XML notation for expressing geographic
annotation and visualization within Internet-based, two-dimensional maps and
three-dimensional Earth browsers.
"""

from __future__ import unicode_literals

from xml.etree import ElementTree

from plaso.output import interface
from plaso.output import manager
from plaso.output import rawpy


class KMLOutputModule(interface.LinearOutputModule):
  """Output module for a Keyhole Markup Language (KML) XML file."""

  NAME = 'kml'
  DESCRIPTION = 'Saves events with geography data into a KML format.'

  def WriteEventBody(self, event):
    """Writes the body of an event to the output.

    Args:
      event (EventObject): event.
    """
    latitude = getattr(event, 'latitude', None)
    longitude = getattr(event, 'longitude', None)
    if latitude is not None and longitude is not None:
      placemark_xml_element = ElementTree.Element('Placemark')

      name_xml_element = ElementTree.SubElement(placemark_xml_element, 'name')

      name_xml_element.text = 'PLACEHOLDER FOR EVENT IDENTIFIER'

      description_xml_element = ElementTree.SubElement(
          placemark_xml_element, 'description')
      # TODO: move the description formatting into this output module.
      description_xml_element.text = (
          rawpy.NativePythonFormatterHelper.GetFormattedEventObject(event))

      point_xml_element = ElementTree.SubElement(
          placemark_xml_element, 'Point')

      coordinates_xml_element = ElementTree.SubElement(
          point_xml_element, 'coordinates')
      coordinates_xml_element.text = '{0!s},{1!s}'.format(longitude, latitude)

      # Note that ElementTree.tostring() will appropriately escape
      # the input data.
      xml_string = ElementTree.tostring(placemark_xml_element)

      output_text = xml_string.encode(self._output_mediator.encoding)
      self._output_writer.Write(output_text)

  def WriteHeader(self):
    """Writes the header to the output."""
    xml_string = (
        '<?xml version="1.0" encoding="{0:s}"?>'
        '<kml xmlns="http://www.opengis.net/kml/2.2"><Document>'.format(
            self._output_mediator.encoding))
    output_text = xml_string.encode(self._output_mediator.encoding)
    self._output_writer.Write(output_text)

  def WriteFooter(self):
    """Writes the footer to the output."""
    xml_string = '</Document></kml>'
    output_text = xml_string.encode(self._output_mediator.encoding)
    self._output_writer.Write(output_text)


manager.OutputManager.RegisterOutput(KMLOutputModule)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests the engine."""

from __future__ import unicode_literals

import unittest

try:
  from guppy import hpy
except ImportError:
  hpy = None

from artifacts import reader as artifacts_reader
from artifacts import registry as artifacts_registry
from dfvfs.helpers import fake_file_system_builder
from dfvfs.helpers import file_system_searcher
from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory
from dfvfs.path import path_spec
from dfvfs.resolver import context
from dfvfs.vfs import file_system as dfvfs_file_system

from plaso.engine import configurations
from plaso.engine import engine
from plaso.lib import definitions

from tests import test_lib as shared_test_lib


class TestEngine(engine.BaseEngine):
  """Class that defines the processing engine for testing."""

  def __init__(self):
    """Initialize a test engine object."""
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    test_file_path = shared_test_lib.GetTestFilePath(['SOFTWARE'])
    file_system_builder.AddFileReadData(
        '/Windows/System32/config/SOFTWARE', test_file_path)
    test_file_path = shared_test_lib.GetTestFilePath(['SYSTEM'])
    file_system_builder.AddFileReadData(
        '/Windows/System32/config/SYSTEM', test_file_path)

    super(TestEngine, self).__init__()
    self._file_system = file_system_builder.file_system
    self._mount_point = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_FAKE, location='/')

  def GetSourceFileSystem(self, source_path_spec, resolver_context=None):
    """Retrieves the file system of the source.

    Args:
      source_path_spec (dfvfs.PathSpec): path specifications of the sources
          to process.
      resolver_context (dfvfs.Context): resolver context.

    Returns:
      tuple: containing:

        dfvfs.FileSystem: file system
        path.PathSpec: mount point path specification. The mount point path
            specification refers to either a directory or a volume on a storage
            media device or image. It is needed by the dfVFS file system
            searcher (FileSystemSearcher) to indicate the base location of
            the file system
    """
    self._file_system.Open(self._mount_point)
    return self._file_system, self._mount_point


class BaseEngineTest(shared_test_lib.BaseTestCase):
  """Tests for the engine object."""

  # pylint: disable=protected-access

  @shared_test_lib.skipUnlessHasTestFile(['ímynd.dd'])
  def testDetermineOperatingSystem(self):
    """Tests the _DetermineOperatingSystem function."""
    test_engine = engine.BaseEngine()

    source_path = self._GetTestFilePath(['ímynd.dd'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=source_path)
    source_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, location='/',
        parent=os_path_spec)

    file_system, mount_point = test_engine.GetSourceFileSystem(source_path_spec)

    searcher = file_system_searcher.FileSystemSearcher(
        file_system, mount_point)

    operating_system = test_engine._DetermineOperatingSystem(searcher)
    self.assertEqual(operating_system, definitions.OPERATING_SYSTEM_UNKNOWN)

  def testStartStopProfiling(self):
    """Tests the _StartProfiling and _StopProfiling functions."""
    with shared_test_lib.TempDirectory() as temp_directory:
      configuration = configurations.ProcessingConfiguration()
      configuration.profiling.directory = temp_directory
      configuration.profiling.profilers = set([
          'memory', 'parsers', 'processing', 'serializers', 'storage',
          'task_queue'])

      test_engine = engine.BaseEngine()

      test_engine._StartProfiling(None)

      test_engine._StartProfiling(configuration.profiling)
      test_engine._StopProfiling()

  def testCreateSession(self):
    """Tests the CreateSession function."""
    test_engine = engine.BaseEngine()

    session = test_engine.CreateSession()
    self.assertIsNotNone(session)

  @shared_test_lib.skipUnlessHasTestFile(['ímynd.dd'])
  def testGetSourceFileSystem(self):
    """Tests the GetSourceFileSystem function."""
    test_engine = engine.BaseEngine()

    source_path = self._GetTestFilePath(['ímynd.dd'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=source_path)
    source_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, location='/',
        parent=os_path_spec)

    resolver_context = context.Context()
    test_file_system, test_mount_point = test_engine.GetSourceFileSystem(
        source_path_spec, resolver_context=resolver_context)

    self.assertIsNotNone(test_file_system)
    self.assertIsInstance(test_file_system, dfvfs_file_system.FileSystem)

    self.assertIsNotNone(test_mount_point)
    self.assertIsInstance(test_mount_point, path_spec.PathSpec)

    test_file_system.Close()

    with self.assertRaises(RuntimeError):
      test_engine.GetSourceFileSystem(None)

  @shared_test_lib.skipUnlessHasTestFile(['artifacts'])
  @shared_test_lib.skipUnlessHasTestFile(['SOFTWARE'])
  @shared_test_lib.skipUnlessHasTestFile(['SYSTEM'])
  def testPreprocessSources(self):
    """Tests the PreprocessSources function."""
    registry = artifacts_registry.ArtifactDefinitionsRegistry()
    reader = artifacts_reader.YamlArtifactsReader()
    path = shared_test_lib.GetTestFilePath(['artifacts'])
    registry.ReadFromDirectory(reader, path)

    test_engine = TestEngine()

    source_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_FAKE, location='/')

    test_engine.PreprocessSources(registry, [source_path_spec])

    operating_system = test_engine.knowledge_base.GetValue('operating_system')
    self.assertEqual(operating_system, 'Windows')

    test_engine.PreprocessSources(registry, [None])

  def testSupportsGuppyMemoryProfiling(self):
    """Tests the SupportsGuppyMemoryProfiling function."""
    test_engine = engine.BaseEngine()

    expected_result = hpy is not None
    result = test_engine.SupportsGuppyMemoryProfiling()
    self.assertEqual(result, expected_result)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Parser for Safari Binary Cookie files."""

from __future__ import unicode_literals

import construct

from dfdatetime import cocoa_time as dfdatetime_cocoa_time
from dfdatetime import semantic_time as dfdatetime_semantic_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import errors
from plaso.lib import definitions
from plaso.lib import specification
# Need to register cookie plugins.
from plaso.parsers import cookie_plugins  # pylint: disable=unused-import
from plaso.parsers import interface
from plaso.parsers import manager
from plaso.parsers.cookie_plugins import manager as cookie_plugins_manager


class SafariBinaryCookieEventData(events.EventData):
  """Safari binary cookie event data.

  Attributes:
    cookie_name (str): cookie name.
    cookie_value (str): cookie value.
    flags (int): cookie flags.
    path (str): path of the cookie.
    url (str): URL where this cookie is valid.
  """

  DATA_TYPE = 'safari:cookie:entry'

  def __init__(self):
    """Initializes event data."""
    super(SafariBinaryCookieEventData, self).__init__(data_type=self.DATA_TYPE)
    self.cookie_name = None
    self.cookie_value = None
    self.flags = None
    self.path = None
    self.url = None


class BinaryCookieParser(interface.FileObjectParser):
  """Parser for Safari Binary Cookie files."""

  NAME = 'binary_cookies'
  DESCRIPTION = 'Parser for Safari Binary Cookie files.'

  _FILE_HEADER = construct.Struct(
      'file_header',
      construct.Bytes('signature', 4),
      construct.UBInt32('number_of_pages'),
      construct.Array(
          lambda ctx: ctx.number_of_pages,
          construct.UBInt32('page_sizes')))

  _COOKIE_RECORD = construct.Struct(
      'cookie_record',
      construct.ULInt32('size'),
      construct.Bytes('unknown_1', 4),
      construct.ULInt32('flags'),
      construct.Bytes('unknown_2', 4),
      construct.ULInt32('url_offset'),
      construct.ULInt32('name_offset'),
      construct.ULInt32('path_offset'),
      construct.ULInt32('value_offset'),
      construct.Bytes('end_of_cookie', 8),
      construct.LFloat64('expiration_date'),
      construct.LFloat64('creation_date'))

  _PAGE_HEADER = construct.Struct(
      'page_header',
      construct.Bytes('header', 4),
      construct.ULInt32('number_of_records'),
      construct.Array(
          lambda ctx: ctx.number_of_records,
          construct.ULInt32('offsets')))

  def __init__(self):
    """Initializes a parser object."""
    super(BinaryCookieParser, self).__init__()
    self._cookie_plugins = (
        cookie_plugins_manager.CookiePluginsManager.GetPlugins())

  def _ParseCookieRecord(self, parser_mediator, page_data, page_offset):
    """Parses a cookie record

    Args:
      parser_mediator (ParserMediator): parser mediator.
      page_data (bytes): page data.
      page_offset (int): offset of the cookie record relative to the start
          of the page.
    """
    try:
      cookie = self._COOKIE_RECORD.parse(page_data[page_offset:])
    except construct.FieldError:
      message = 'Unable to read cookie record at offset: {0:d}'.format(
          page_offset)
      parser_mediator.ProduceExtractionError(message)
      return

    # The offset is determined by the range between the start of the current
    # offset until the start of the next offset. Thus we need to determine
    # the proper ordering of the offsets, since they are not always in the
    # same ordering.
    offset_dict = {
        cookie.url_offset: 'url', cookie.name_offset: 'name',
        cookie.value_offset: 'value', cookie.path_offset: 'path'}

    offsets = sorted(offset_dict.keys())
    offsets.append(cookie.size + page_offset)

    # TODO: Find a better approach to parsing the data than this.
    data_dict = {}
    for current_offset in range(0, len(offsets) - 1):
      # Get the current offset and the offset for the next entry.
      start, end = offsets[current_offset:current_offset+2]
      value = offset_dict.get(offsets[current_offset])
      # Read the data.
      data_all = page_data[start + page_offset:end + page_offset]
      data, _, _ = data_all.partition(b'\x00')
      data_dict[value] = data

    event_data = SafariBinaryCookieEventData()
    event_data.cookie_name = data_dict.get('name')
    event_data.cookie_value = data_dict.get('value')
    event_data.flags = cookie.flags
    event_data.path = data_dict.get('path')
    event_data.url = data_dict.get('url')

    if cookie.creation_date:
      date_time = dfdatetime_cocoa_time.CocoaTime(
          timestamp=cookie.creation_date)
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_CREATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)

    if cookie.expiration_date:
      date_time = dfdatetime_cocoa_time.CocoaTime(
          timestamp=cookie.expiration_date)
    else:
      date_time = dfdatetime_semantic_time.SemanticTime('Not set')

    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_EXPIRATION)
    parser_mediator.ProduceEventWithEventData(event, event_data)

    for plugin in self._cookie_plugins:
      if parser_mediator.abort:
        break

      if event_data.cookie_name != plugin.COOKIE_NAME:
        continue

      try:
        plugin.UpdateChainAndProcess(
            parser_mediator, cookie_name=event_data.cookie_name,
            cookie_data=event_data.cookie_value, url=event_data.url)

      except Exception as exception:  # pylint: disable=broad-except
        parser_mediator.ProduceExtractionError(
            'plugin: {0:s} unable to parse cookie with error: {1!s}'.format(
                plugin.NAME, exception))

  def _ParsePage(self, parser_mediator, page_number, page_data):
    """Parses a page.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      page_number (int): page number.
      page_data (bytes): page data.
    """
    try:
      page_header = self._PAGE_HEADER.parse(page_data)
    except construct.FieldError:
      # TODO: add offset
      parser_mediator.ProduceExtractionError(
          'unable to read header of page: {0:d} at offset: 0x{1:08x}'.format(
              page_number, 0))
      return

    for page_offset in page_header.offsets:
      if parser_mediator.abort:
        break

      self._ParseCookieRecord(parser_mediator, page_data, page_offset)

    # TODO: check footer.

  @classmethod
  def GetFormatSpecification(cls):
    """Retrieves the format specification for parser selection.

    Returns:
      FormatSpecification: format specification.
    """
    format_specification = specification.FormatSpecification(cls.NAME)
    format_specification.AddNewSignature(b'cook\x00', offset=0)
    return format_specification

  def ParseFileObject(self, parser_mediator, file_object, **kwargs):
    """Parses a Safari binary cookie file-like object.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      file_object (dfvfs.FileIO): file-like object to be parsed.

    Raises:
      UnableToParseFile: when the file cannot be parsed, this will signal
          the event extractor to apply other parsers.
    """
    try:
      file_header = self._FILE_HEADER.parse_stream(file_object)
    except (IOError, construct.ArrayError, construct.FieldError) as exception:
      parser_mediator.ProduceExtractionError(
          'unable to read file header with error: {0!s}.'.format(exception))
      raise errors.UnableToParseFile()

    if file_header.signature != b'cook':
      parser_mediator.ProduceExtractionError('unsupported file signature.')
      raise errors.UnableToParseFile()

    for index, page_size in enumerate(file_header.page_sizes):
      if parser_mediator.abort:
        break

      page_data = file_object.read(page_size)
      if len(page_data) != page_size:
        parser_mediator.ProduceExtractionError(
            'unable to read page: {0:d}'.format(index))
        break

      self._ParsePage(parser_mediator, index, page_data)


manager.ParsersManager.RegisterParser(BinaryCookieParser)



# -*- coding: utf-8 -*-
"""This file contains the Windows NT Language identifiers."""

from __future__ import unicode_literals


LANGUAGE_IDENTIFIERS = {
    'af': [0x0036, 'Afrikaans'],
    'af-ZA': [0x0436, 'Afrikaans, South Africa'],
    'am': [0x005e, 'Amharic'],
    'am-ET': [0x045e, 'Amharic, Ethiopia'],
    'ar': [0x0001, 'Arabic'],
    'ar-AE': [0x3801, 'Arabic, United Arab Emirates'],
    'ar-BH': [0x3c01, 'Arabic, Bahrain'],
    'ar-DZ': [0x1401, 'Arabic, Algeria'],
    'ar-EG': [0x0c01, 'Arabic, Egypt'],
    'ar-IQ': [0x0801, 'Arabic, Iraq'],
    'ar-JO': [0x2c01, 'Arabic, Jordan'],
    'ar-KW': [0x3401, 'Arabic, Kuwait'],
    'ar-LB': [0x3001, 'Arabic, Lebanon'],
    'ar-LY': [0x1001, 'Arabic, Libyan Arab Jamahiriya'],
    'ar-MA': [0x1801, 'Arabic, Morocco'],
    'arn': [0x007a, 'Mapudungun'],
    'arn-CL': [0x047a, 'Mapudungun, Chile'],
    'ar-OM': [0x2001, 'Arabic, Oman'],
    'ar-QA': [0x4001, 'Arabic, Qatar'],
    'ar-SA': [0x0401, 'Arabic, Saudi Arabia'],
    'ar-SY': [0x2801, 'Arabic, Syrian Arab Republic'],
    'ar-TN': [0x1c01, 'Arabic, Tunisia'],
    'ar-YE': [0x2401, 'Arabic, Yemen'],
    'as': [0x004d, 'Assamese'],
    'as-IN': [0x044d, 'Assamese, India'],
    'az': [0x002c, 'Azerbaijani'],
    'az-Cyrl': [0x742c, 'Azerbaijani, Cyrillic'],
    'az-Cyrl-AZ': [0x082c, 'Azerbaijani, Cyrillic, Azerbaijan'],
    'az-Latn': [0x782c, 'Azerbaijani, Latin'],
    'az-Latn-AZ': [0x042c, 'Azerbaijani, Latin, Azerbaijan'],
    'ba': [0x006d, 'Bashkir'],
    'ba-RU': [0x046d, 'Bashkir, Russian Federation'],
    'be': [0x0023, 'Belarusian'],
    'be-BY': [0x0423, 'Belarusian, Belarus'],
    'bg': [0x0002, 'Bulgarian'],
    'bg-BG': [0x0402, 'Bulgarian, Bulgaria'],
    'bin-NG': [0x0466, 'Bini, Nigeria'],
    'bn': [0x0045, 'Bengali'],
    'bn-BD': [0x0845, 'Bengali, Bangladesh'],
    'bn-IN': [0x0445, 'Bengali, India'],
    'bo': [0x0051, 'Tibetan'],
    'bo-BT': [0x0851, 'Tibetan, Bhutan'],
    'bo-CN': [0x0451, 'Tibetan, China'],
    'br': [0x007e, 'Breton'],
    'br-FR': [0x047e, 'Breton, France'],
    'bs': [0x781a, 'Bosnian'],
    'bs-Cyrl': [0x641a, 'Bosnian, Cyrillic'],
    'bs-Cyrl-BA': [0x201a, 'Bosnian, Cyrillic, Bosnia and Herzegovina'],
    'bs-Latn': [0x681a, 'Bosnian, Latin'],
    'bs-Latn-BA': [0x141a, 'Bosnian, Latin, Bosnia and Herzegovina'],
    'ca': [0x0003, 'Catalan'],
    'ca-ES': [0x0403, 'Catalan, Spain'],
    'chr-US': [0x045c, 'Cherokee, United States'],
    'co': [0x0083, 'Corsican'],
    'co-FR': [0x0483, 'Corsican, France'],
    'cs': [0x0005, 'Czech'],
    'cs-CZ': [0x0405, 'Czech, Czech Republic'],
    'cy': [0x0052, 'Welsh'],
    'cy-GB': [0x0452, 'Welsh, United Kingdom'],
    'da': [0x0006, 'Danish'],
    'da-DK': [0x0406, 'Danish, Denmark'],
    'de': [0x0007, 'German'],
    'de-AT': [0x0c07, 'German, Austria'],
    'de-CH': [0x0807, 'German, Switzerland'],
    'de-DE': [0x0407, 'German, Germany'],
    'de-LI': [0x1407, 'German, Liechtenstein'],
    'de-LU': [0x1007, 'German, Luxembourg'],
    'dsb': [0x7c2e, 'Lower Sorbian'],
    'dsb-DE': [0x082e, 'Lower Sorbian, Germany'],
    'dv': [0x0065, 'Dhivehi'],
    'dv-MV': [0x0465, 'Dhivehi, Maldives'],
    'el': [0x0008, 'Modern Greek (1453 and later)'],
    'el-GR': [0x0408, 'Modern Greek (1453-), Greece'],
    'en': [0x0009, 'English'],
    'en-AU': [0x0c09, 'English, Australia'],
    'en-BZ': [0x2809, 'English, Belize'],
    'en-CA': [0x1009, 'English, Canada'],
    'en-CB': [0x2409, 'English'],
    'en-GB': [0x0809, 'English, United Kingdom'],
    'en-HK': [0x3c09, 'English, Hong Kong'],
    'en-ID': [0x3809, 'English, Indonesia'],
    'en-IE': [0x1809, 'English, Ireland'],
    'en-IN': [0x4009, 'English, India'],
    'en-JM': [0x2009, 'English, Jamaica'],
    'en-MY': [0x4409, 'English, Malaysia'],
    'en-NZ': [0x1409, 'English, New Zealand'],
    'en-PH': [0x3409, 'English, Philippines'],
    'en-SG': [0x4809, 'English, Singapore'],
    'en-TT': [0x2c09, 'English, Trinidad and Tobago'],
    'en-US': [0x0409, 'English, United States'],
    'en-ZA': [0x1c09, 'English, South Africa'],
    'en-ZW': [0x3009, 'English, Zimbabwe'],
    'es': [0x000a, 'Spanish'],
    'es-AR': [0x2c0a, 'Spanish, Argentina'],
    'es-BO': [0x400a, 'Spanish, Bolivia'],
    'es-CL': [0x340a, 'Spanish, Chile'],
    'es-CO': [0x240a, 'Spanish, Colombia'],
    'es-CR': [0x140a, 'Spanish, Costa Rica'],
    'es-DO': [0x1c0a, 'Spanish, Dominican Republic'],
    'es-EC': [0x300a, 'Spanish, Ecuador'],
    'es-ES': [0x0c0a, 'Spanish, Spain'],
    'es-ES_tradnl': [0x040a, 'Spanish'],
    'es-GT': [0x100a, 'Spanish, Guatemala'],
    'es-HN': [0x480a, 'Spanish, Honduras'],
    'es-MX': [0x080a, 'Spanish, Mexico'],
    'es-NI': [0x4c0a, 'Spanish, Nicaragua'],
    'es-PA': [0x180a, 'Spanish, Panama'],
    'es-PE': [0x280a, 'Spanish, Peru'],
    'es-PR': [0x500a, 'Spanish, Puerto Rico'],
    'es-PY': [0x3c0a, 'Spanish, Paraguay'],
    'es-SV': [0x440a, 'Spanish, El Salvador'],
    'es-US': [0x540a, 'Spanish, United States'],
    'es-UY': [0x380a, 'Spanish, Uruguay'],
    'es-VE': [0x200a, 'Spanish, Venezuela'],
    'et': [0x0025, 'Estonian'],
    'et-EE': [0x0425, 'Estonian, Estonia'],
    'eu': [0x002d, 'Basque'],
    'eu-ES': [0x042d, 'Basque, Spain'],
    'fa': [0x0029, 'Persian'],
    'fa-IR': [0x0429, 'Persian, Islamic Republic of Iran'],
    'fi': [0x000b, 'Finnish'],
    'fi-FI': [0x040b, 'Finnish, Finland'],
    'fil': [0x0064, 'Filipino'],
    'fil-PH': [0x0464, 'Filipino, Philippines'],
    'fo': [0x0038, 'Faroese'],
    'fo-FO': [0x0438, 'Faroese, Faroe Islands'],
    'fr': [0x000c, 'French'],
    'fr-BE': [0x080c, 'French, Belgium'],
    'fr-CA': [0x0c0c, 'French, Canada'],
    'fr-CG': [0x240c, 'French, Congo'],
    'fr-CH': [0x100c, 'French, Switzerland'],
    'fr-CI': [0x300c, 'French, Côte d\'Ivoire'],
    'fr-CM': [0x2c0c, 'French, Cameroon'],
    'fr-FR': [0x040c, 'French, France'],
    'fr-HT': [0x3c0c, 'French, Haiti'],
    'fr-LU': [0x140c, 'French, Luxembourg'],
    'fr-MA': [0x380c, 'French, Morocco'],
    'fr-MC': [0x180c, 'French, Monaco'],
    'fr-ML': [0x340c, 'French, Mali'],
    'fr-RE': [0x200c, 'French, Réunion'],
    'fr-SN': [0x280c, 'French, Senegal'],
    'fr-West': [0x1c0c, 'French'],
    'fuv-NG': [0x0467, 'Nigerian Fulfulde, Nigeria'],
    'fy': [0x0062, 'Western Frisian'],
    'fy-NL': [0x0462, 'Western Frisian, Netherlands'],
    'ga': [0x003c, 'Irish'],
    'ga-IE': [0x083c, 'Irish, Ireland'],
    'gaz-ET': [0x0472, 'West Central Oromo, Ethiopia'],
    'gd': [0x0091, 'Scottish Gaelic'],
    'gd-GB': [0x0491, 'Scottish Gaelic, United Kingdom'],
    'gl': [0x0056, 'Galician'],
    'gl-ES': [0x0456, 'Galician, Spain'],
    'gn-PY': [0x0474, 'Guarani, Paraguay'],
    'gsw': [0x0084, 'Swiss German'],
    'gsw-FR': [0x0484, 'Swiss German, France'],
    'gu': [0x0047, 'Gujarati'],
    'gu-IN': [0x0447, 'Gujarati, India'],
    'ha': [0x0068, 'Hausa'],
    'ha-Latn': [0x7c68, 'Hausa, Latin'],
    'ha-Latn-NG': [0x0468, 'Hausa, Latin, Nigeria'],
    'haw-US': [0x0475, 'Hawaiian, United States'],
    'he': [0x000d, 'Hebrew'],
    'he-IL': [0x040d, 'Hebrew, Israel'],
    'hi': [0x0039, 'Hindi'],
    'hi-IN': [0x0439, 'Hindi, India'],
    'hr': [0x001a, 'Croatian'],
    'hr-BA': [0x101a, 'Croatian, Bosnia and Herzegovina'],
    'hr-HR': [0x041a, 'Croatian, Croatia'],
    'hsb': [0x002e, 'Upper Sorbian'],
    'hu': [0x000e, 'Hungarian'],
    'hu-HU': [0x040e, 'Hungarian, Hungary'],
    'hy': [0x002b, 'Armenian'],
    'hy-AM': [0x042b, 'Armenian, Armenia'],
    'ibb-NG': [0x0469, 'Ibibio, Nigeria'],
    'id': [0x0021, 'Indonesian'],
    'id-ID': [0x0421, 'Indonesian, Indonesia'],
    'ig': [0x0070, 'Igbo'],
    'ig-NG': [0x0470, 'Igbo, Nigeria'],
    'ii': [0x0078, 'Sichuan Yi'],
    'ii-CN': [0x0478, 'Sichuan Yi, China'],
    'is': [0x000f, 'Icelandic'],
    'is-IS': [0x040f, 'Icelandic, Iceland'],
    'it': [0x0010, 'Italian'],
    'it-CH': [0x0810, 'Italian, Switzerland'],
    'it-IT': [0x0410, 'Italian, Italy'],
    'iu': [0x005d, 'Inuktitut'],
    'iu-Cans': [0x785d, 'Inuktitut, Unified Canadian Aboriginal Syllabics'],
    'iu-Cans-CA': [
        0x045d, 'Inuktitut, Unified Canadian Aboriginal Syllabics, Canada'],
    'iu-Latn': [0x7c5d, 'Inuktitut, Latin'],
    'iu-Latn-CA': [0x085d, 'Inuktitut, Latin, Canada'],
    'ja': [0x0011, 'Japanese'],
    'ja-JP': [0x0411, 'Japanese, Japan'],
    'ka': [0x0037, 'Georgian'],
    'ka-GE': [0x0437, 'Georgian, Georgia'],
    'kk': [0x003f, 'Kazakh'],
    'kk-KZ': [0x043f, 'Kazakh, Kazakhstan'],
    'kl': [0x006f, 'Kalaallisut'],
    'kl-GL': [0x046f, 'Kalaallisut, Greenland'],
    'km': [0x0053, 'Central Khmer'],
    'km-KH': [0x0453, 'Central Khmer, Cambodia'],
    'kn': [0x004b, 'Kannada'],
    'kn-IN': [0x044b, 'Kannada, India'],
    'ko': [0x0012, 'Korean'],
    'kok': [0x0057, 'Konkani (macrolanguage)'],
    'kok-IN': [0x0457, 'Konkani (macrolanguage), India'],
    'ko-KR': [0x0412, 'Korean, Republic of Korea'],
    'kr-NG': [0x0471, 'Kanuri, Nigeria'],
    'ky': [0x0040, 'Kirghiz'],
    'ky-KG': [0x0440, 'Kirghiz, Kyrgyzstan'],
    'lb': [0x006e, 'Luxembourgish'],
    'lb-LU': [0x046e, 'Luxembourgish, Luxembourg'],
    'lo': [0x0054, 'Lao'],
    'lo-LA': [0x0454, 'Lao, Lao People\'s Democratic Republic'],
    'lt': [0x0027, 'Lithuanian'],
    'lt-LT': [0x0427, 'Lithuanian, Lithuania'],
    'lv': [0x0026, 'Latvian'],
    'lv-LV': [0x0426, 'Latvian, Latvia'],
    'mi': [0x0081, 'Maori'],
    'mi-NZ': [0x0481, 'Maori, New Zealand'],
    'mk': [0x002f, 'Macedonian'],
    'mk-MK': [
        0x042f, 'Macedonian, The Former Yugoslav Republic of Macedonia'],
    'ml': [0x004c, 'Malayalam'],
    'ml-IN': [0x044c, 'Malayalam, India'],
    'mn': [0x0050, 'Mongolian'],
    'mn-Cyrl': [0x7850, 'Mongolian, Cyrillic'],
    'mni': [0x0458, 'Manipuri'],
    'mn-MN': [0x0450, 'Mongolian, Mongolia'],
    'mn-Mong': [0x7c50, 'Mongolian, Mongolian'],
    'mn-Mong-CN': [0x0850, 'Mongolian, Mongolian, China'],
    'moh': [0x007c, 'Mohawk'],
    'moh-CA': [0x047c, 'Mohawk, Canada'],
    'mr': [0x004e, 'Marathi'],
    'mr-IN': [0x044e, 'Marathi, India'],
    'ms': [0x003e, 'Malay (macrolanguage)'],
    'ms-BN': [0x083e, 'Malay (macrolanguage), Brunei Darussalam'],
    'ms-MY': [0x043e, 'Malay (macrolanguage), Malaysia'],
    'mt': [0x003a, 'Maltese'],
    'mt-MT': [0x043a, 'Maltese, Malta'],
    'my-MM': [0x0455, 'Burmese, Myanmar'],
    'nb': [0x7c14, 'Norwegian Bokmål'],
    'nb-NO': [0x0414, 'Norwegian Bokmål, Norway'],
    'ne': [0x0061, 'Nepali'],
    'ne-IN': [0x0861, 'Nepali, India'],
    'ne-NP': [0x0461, 'Nepali, Nepal'],
    'nl': [0x0013, 'Dutch'],
    'nl-BE': [0x0813, 'Dutch, Belgium'],
    'nl-NL': [0x0413, 'Dutch, Netherlands'],
    'nn': [0x7814, 'Norwegian Nynorsk'],
    'nn-NO': [0x0814, 'Norwegian Nynorsk, Norway'],
    'no': [0x0014, 'Norwegian'],
    'nso': [0x006c, 'Pedi'],
    'nso-ZA': [0x046c, 'Pedi, South Africa'],
    'oc': [0x0082, 'Occitan (post 1500)'],
    'oc-FR': [0x0482, 'Occitan (post 1500), France'],
    'or': [0x0048, 'Oriya'],
    'or-IN': [0x0448, 'Oriya, India'],
    'pa': [0x0046, 'Panjabi'],
    'pa-IN': [0x0446, 'Panjabi, India'],
    'pap-AN': [0x0479, 'Papiamento, Netherlands Antilles'],
    'pa-PK': [0x0846, 'Panjabi, Pakistan'],
    'pl': [0x0015, 'Polish'],
    'pl-PL': [0x0415, 'Polish, Poland'],
    'plt-MG': [0x048d, 'Plateau Malagasy, Madagascar'],
    'prs': [0x008c, 'Dari'],
    'prs-AF': [0x048c, 'Dari, Afghanistan'],
    'ps': [0x0063, 'Pushto'],
    'ps-AF': [0x0463, 'Pushto, Afghanistan'],
    'pt': [0x0016, 'Portuguese'],
    'pt-BR': [0x0416, 'Portuguese, Brazil'],
    'pt-PT': [0x0816, 'Portuguese, Portugal'],
    'qut': [0x0086, 'Guatemala'],
    'qut-GT': [0x0486, 'Guatemala'],
    'quz': [0x006b, 'Cusco Quechua'],
    'quz-BO': [0x046b, 'Cusco Quechua, Bolivia'],
    'quz-EC': [0x086b, 'Cusco Quechua, Ecuador'],
    'quz-PE': [0x0c6b, 'Cusco Quechua, Peru'],
    'rm': [0x0017, 'Romansh'],
    'rm-CH': [0x0417, 'Romansh, Switzerland'],
    'ro': [0x0018, 'Romanian'],
    'ro-MO': [0x0818, 'Romanian, Macao'],
    'ro-RO': [0x0418, 'Romanian, Romania'],
    'ru': [0x0019, 'Russian'],
    'ru-MO': [0x0819, 'Russian, Macao'],
    'ru-RU': [0x0419, 'Russian, Russian Federation'],
    'rw': [0x0087, 'Kinyarwanda'],
    'rw-RW': [0x0487, 'Kinyarwanda, Rwanda'],
    'sa': [0x004f, 'Sanskrit'],
    'sah': [0x0085, 'Yakut'],
    'sah-RU': [0x0485, 'Yakut, Russian Federation'],
    'sa-IN': [0x044f, 'Sanskrit, India'],
    'sd-IN': [0x0459, 'Sindhi, India'],
    'sd-PK': [0x0859, 'Sindhi, Pakistan'],
    'se': [0x003b, 'Northern Sami'],
    'se-FI': [0x0c3b, 'Northern Sami, Finland'],
    'se-NO': [0x043b, 'Northern Sami, Norway'],
    'se-SE': [0x083b, 'Northern Sami, Sweden'],
    'si': [0x005b, 'Sinhala'],
    'si-LK': [0x045b, 'Sinhala, Sri Lanka'],
    'sk': [0x001b, 'Slovak'],
    'sk-SK': [0x041b, 'Slovak, Slovakia'],
    'sl': [0x0024, 'Slovenian'],
    'sl-SI': [0x0424, 'Slovenian, Slovenia'],
    'sma': [0x783b, 'Southern Sami'],
    'sma-NO': [0x183b, 'Southern Sami, Norway'],
    'sma-SE': [0x1c3b, 'Southern Sami, Sweden'],
    'smj': [0x7c3b, 'Lule Sami'],
    'smj-NO': [0x103b, 'Lule Sami, Norway'],
    'smj-SE': [0x143b, 'Lule Sami, Sweden'],
    'smn': [0x703b, 'Inari Sami'],
    'smn-FI': [0x243b, 'Inari Sami, Finland'],
    'sms': [0x743b, 'Skolt Sami'],
    'sms-FI': [0x203b, 'Skolt Sami, Finland'],
    'so-SO': [0x0477, 'Somali, Somalia'],
    'sq': [0x001c, 'Albanian'],
    'sq-AL': [0x041c, 'Albanian, Albania'],
    'sr': [0x7c1a, 'Serbian'],
    'sr-Cyrl': [0x6c1a, 'Serbian, Cyrillic'],
    'sr-Cyrl-BA': [0x1c1a, 'Serbian, Cyrillic, Bosnia and Herzegovina'],
    'sr-Cyrl-CS': [0x0c1a, 'Serbian, Cyrillic, Serbia and Montenegro'],
    'sr-Cyrl-ME': [0x301a, 'Serbian, Cyrillic, Montenegro'],
    'sr-Cyrl-RS': [0x281a, 'Serbian, Cyrillic, Serbia'],
    'sr-Latn': [0x701a, 'Serbian, Latin'],
    'sr-Latn-BA': [0x181a, 'Serbian, Latin, Bosnia and Herzegovina'],
    'sr-Latn-CS': [0x081a, 'Serbian, Latin, Serbia and Montenegro'],
    'sr-Latn-ME': [0x2c1a, 'Serbian, Latin, Montenegro'],
    'sr-Latn-RS': [0x241a, 'Serbian, Latin, Serbia'],
    'st-ZA': [0x0430, 'Southern Sotho, South Africa'],
    'sv': [0x001d, 'Swedish'],
    'sv-FI': [0x081d, 'Swedish, Finland'],
    'sv-SE': [0x041d, 'Swedish, Sweden'],
    'sw': [0x0041, 'Swahili (macrolanguage)'],
    'sw-KE': [0x0441, 'Swahili (macrolanguage), Kenya'],
    'syr': [0x005a, 'Syriac'],
    'syr-SY': [0x045a, 'Syriac, Syrian Arab Republic'],
    'ta': [0x0049, 'Tamil'],
    'ta-IN': [0x0449, 'Tamil, India'],
    'te': [0x004a, 'Telugu'],
    'te-IN': [0x044a, 'Telugu, India'],
    'tg': [0x0028, 'Tajik'],
    'tg-Cyrl': [0x7c28, 'Tajik, Cyrillic'],
    'tg-Cyrl-TJ': [0x0428, 'Tajik, Cyrillic, Tajikistan'],
    'th': [0x001e, 'Thai'],
    'th-TH': [0x041e, 'Thai, Thailand'],
    'ti-ER': [0x0473, 'Tigrinya, Eritrea'],
    'ti-ET': [0x0873, 'Tigrinya, Ethiopia'],
    'tk': [0x0042, 'Turkmen'],
    'tk-TM': [0x0442, 'Turkmen, Turkmenistan'],
    'tmz': [0x045f, 'Tamanaku'],
    'tmz-MA': [0x0c5f, 'Tamanaku, Morocco'],
    'tn': [0x0032, 'Tswana'],
    'tn-ZA': [0x0432, 'Tswana, South Africa'],
    'tr': [0x001f, 'Turkish'],
    'tr-TR': [0x041f, 'Turkish, Turkey'],
    'ts-ZA': [0x0431, 'Tsonga, South Africa'],
    'tt': [0x0044, 'Tatar'],
    'tt-RU': [0x0444, 'Tatar, Russian Federation'],
    'tzm': [0x005f, 'Central Atlas Tamazight'],
    'tzm-Latn': [0x7c5f, 'Central Atlas Tamazight, Latin'],
    'tzm-Latn-DZ': [0x085f, 'Central Atlas Tamazight, Latin, Algeria'],
    'ug': [0x0080, 'Uighur'],
    'ug-CN': [0x0480, 'Uighur, China'],
    'uk': [0x0022, 'Ukrainian'],
    'uk-UA': [0x0422, 'Ukrainian, Ukraine'],
    'ur': [0x0020, 'Urdu'],
    'ur-IN': [0x0820, 'Urdu, India'],
    'ur-PK': [0x0420, 'Urdu, Pakistan'],
    'uz': [0x0043, 'Uzbek'],
    'uz-Cyrl': [0x7843, 'Uzbek, Cyrillic'],
    'uz-Cyrl-UZ': [0x0843, 'Uzbek, Cyrillic, Uzbekistan'],
    'uz-Latn': [0x7c43, 'Uzbek, Latin'],
    'uz-Latn-UZ': [0x0443, 'Uzbek, Latin, Uzbekistan'],
    'ven-ZA': [0x0433, 'South Africa'],
    'vi': [0x002a, 'Vietnamese'],
    'vi-VN': [0x042a, 'Vietnamese, Viet Nam'],
    'wen-DE': [0x042e, 'Sorbian languages, Germany'],
    'wo': [0x0088, 'Wolof'],
    'wo-SN': [0x0488, 'Wolof, Senegal'],
    'xh': [0x0034, 'Xhosa'],
    'xh-ZA': [0x0434, 'Xhosa, South Africa'],
    'yo': [0x006a, 'Yoruba'],
    'yo-NG': [0x046a, 'Yoruba, Nigeria'],
    'zh': [0x7804, 'Chinese'],
    'zh-CN': [0x0804, 'Chinese, China'],
    'zh-Hans': [0x0004, 'Chinese, Han (Simplified variant)'],
    'zh-Hant': [0x7c04, 'Chinese, Han (Traditional variant)'],
    'zh-HK': [0x0c04, 'Chinese, Hong Kong'],
    'zh-MO': [0x1404, 'Chinese, Macao'],
    'zh-SG': [0x1004, 'Chinese, Singapore'],
    'zh-TW': [0x0404, 'Chinese, Taiwan, Province of China'],
    'zu': [0x0035, 'Zulu'],
    'zu-ZA': [0x0435, 'Zulu, South Africa'],
}

# Make sure the language identifiers are lower case.
LANGUAGE_IDENTIFIERS = dict(
    (key.lower(), value) for key, value in LANGUAGE_IDENTIFIERS.items())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the SQLite database parser."""

import sys
import unittest

from plaso.parsers import sqlite
# Register all plugins.
from plaso.parsers import sqlite_plugins  # pylint: disable=unused-import

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class SQLiteParserTest(test_lib.ParserTestCase):
  """Tests for the SQLite database parser."""

  # pylint: disable=protected-access

  def testEnablePlugins(self):
    """Tests the EnablePlugins function."""
    parser = sqlite.SQLiteParser()
    parser.EnablePlugins(['chrome_27_history'])

    self.assertIsNotNone(parser)
    self.assertIsNone(parser._default_plugin)
    self.assertNotEqual(parser._plugins, [])
    self.assertEqual(len(parser._plugins), 1)

  @shared_test_lib.skipUnlessHasTestFile(['contacts2.db'])
  def testFileParserChainMaintenance(self):
    """Tests that the parser chain is correctly maintained by the parser."""
    parser = sqlite.SQLiteParser()
    storage_writer = self._ParseFile(['contacts2.db'], parser)

    for event in storage_writer.GetEvents():
      chain = event.parser
      self.assertEqual(1, chain.count('/'))

  @shared_test_lib.skipUnlessHasTestFile(['wal_database.db'])
  @shared_test_lib.skipUnlessHasTestFile(['wal_database.db-wal'])
  def testQueryDatabaseWithWAL(self):
    """Tests the Query function on a database with a WAL file."""
    database_file = self._GetTestFilePath(['wal_database.db'])
    wal_file = self._GetTestFilePath(['wal_database.db-wal'])

    database = sqlite.SQLiteDatabase('wal_database.db')
    with open(database_file, 'rb') as database_file_object:
      with open(wal_file, 'rb') as wal_file_object:
        database.Open(database_file_object, wal_file_object=wal_file_object)

    row_results = []
    for row in database.Query('SELECT * FROM MyTable'):
      # Note that pysqlite does not accept a Unicode string in row['string'] and
      # will raise "IndexError: Index must be int or string".
      # Also, Field3 needs to be converted to a string if Python 2 is used
      # because it is a read-write buffer.
      field3 = row['Field3']
      if sys.version_info[0] < 3:
        field3 = str(field3)
      row_results.append((
          row['Field1'], row['Field2'], field3))

    expected_results = [
        ('Committed Text 1', 1, b'None'),
        ('Committed Text 2', 2, b'None'),
        ('Modified Committed Text 3', 4, b'None'),
        ('Committed Text 4', 5, b'None'),
        ('Committed Text 5', 7, b'None'),
        ('Committed Text 6', 8, b'None'),
        ('Committed Text 7', 9, b'None'),
        ('Unhashable Row 1', 10, b'Binary Text!\x01\x02\x03'),
        ('Unhashable Row 2', 11, b'More Binary Text!\x01\x02\x03'),
        ('New Text 1', 12, b'None'),
        ('New Text 2', 13, b'None')]

    self.assertEqual(expected_results, row_results)

  @shared_test_lib.skipUnlessHasTestFile(['wal_database.db'])
  def testQueryDatabaseWithoutWAL(self):
    """Tests the Query function on a database without a WAL file."""
    database_file = self._GetTestFilePath(['wal_database.db'])

    database = sqlite.SQLiteDatabase('wal_database.db')
    with open(database_file, 'rb') as database_file_object:
      database.Open(database_file_object)

    row_results = []
    for row in database.Query('SELECT * FROM MyTable'):
      # Note that pysqlite does not accept a Unicode string in row['string'] and
      # will raise "IndexError: Index must be int or string".
      # Also, Field3 needs to be converted to a string if Python 2 is used
      # because it is a read-write buffer.
      field3 = row['Field3']
      if sys.version_info[0] < 3:
        field3 = str(field3)
      row_results.append((
          row['Field1'], row['Field2'], field3))

    expected_results = [
        ('Committed Text 1', 1, b'None'),
        ('Committed Text 2', 2, b'None'),
        ('Deleted Text 1', 3, b'None'),
        ('Committed Text 3', 4, b'None'),
        ('Committed Text 4', 5, b'None'),
        ('Deleted Text 2', 6, b'None'),
        ('Committed Text 5', 7, b'None'),
        ('Committed Text 6', 8, b'None'),
        ('Committed Text 7', 9, b'None'),
        ('Unhashable Row 1', 10, b'Binary Text!\x01\x02\x03')]

    self.assertEqual(expected_results, row_results)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The UserAssist Windows Registry event formatter."""

from __future__ import unicode_literals

from plaso.formatters import interface
from plaso.formatters import manager


class UserAssistWindowsRegistryEventFormatter(
    interface.ConditionalEventFormatter):
  """Formatter for an UserAssist Windows Registry event."""

  DATA_TYPE = 'windows:registry:userassist'

  FORMAT_STRING_PIECES = [
      '[{key_path}]',
      'UserAssist entry: {entry_index}',
      'Value name: {value_name}',
      'Count: {number_of_executions}',
      'Application focus count: {application_focus_count}',
      'Application focus duration: {application_focus_duration}']

  FORMAT_STRING_SHORT_PIECES = [
      '{value_name}',
      'Count: {number_of_executions}']

  SOURCE_LONG = 'Registry Key: UserAssist'
  SOURCE_SHORT = 'REG'


manager.FormattersManager.RegisterFormatter(
    UserAssistWindowsRegistryEventFormatter)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the Viper analysis plugin."""

from __future__ import unicode_literals

import unittest

try:
  import mock  # pylint: disable=import-error
except ImportError:
  from unittest import mock

from dfdatetime import posix_time as dfdatetime_posix_time
from dfvfs.path import fake_path_spec

from plaso.analysis import viper
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.lib import timelib

from tests.analysis import test_lib


class MockResponse(dict):
  """A mock object to simulate a response object from the requests library."""

  # Note: that the following functions do not follow the style guide
  # because they are part of the requests response interface.
  # pylint: disable=invalid-name

  def json(self):
    """Provided for compatibility with the requests library."""
    return self

  def raise_for_status(self):
    """Provided for compatibility with the requests library."""
    return


class ViperTest(test_lib.AnalysisPluginTestCase):
  """Tests for the Viper analysis plugin."""

  _EVENT_1_HASH = (
      '2d79fcc6b02a2e183a0cb30e0e25d103f42badda9fbf86bbee06f93aa3855aff')

  _TEST_EVENTS = [{
      'timestamp': timelib.Timestamp.CopyFromString('2015-01-01 17:00:00'),
      'sha256_hash': _EVENT_1_HASH}]

  def _MockPost(self, unused_url, data=None):
    """Mock funtion to simulate a Viper API request.

    Args:
      url (str): URL being requested.
      data (dict[str, object]): simulated form data for the Viper API request.

    Returns:
      MockResponse: mocked response that simulates a real response object
          returned by the requests library from the Viper API.
    """
    sha256_hash = data.get('sha256', None)
    if sha256_hash != self._EVENT_1_HASH:
      self.fail('Unexpected data in request.post().')

    response = MockResponse()
    response['default'] = ({
        'sha1': '13da502ab0d75daca5e5075c60e81bfe3b7a637f',
        'name': 'darkcomet.exe',
        'tags': [
            'rat',
            'darkcomet'],
        'sha512': '7e81e0c4f49f1884ebebdf6e53531e7836721c2ae417'
                  '29cf5bc0340f3369e7d37fe4168a7434b2b0420b299f5c'
                  '1d9a4f482f1bda8e66e40345757d97e5602b2d',
        'created_at': '2015-03-30 23:13:20.595238',
        'crc32': '2238B48E',
        'ssdeep': '12288:D9HFJ9rJxRX1uVVjoaWSoynxdO1FVBaOiRZTERfIhNk'
                  'NCCLo9Ek5C/hlg:NZ1xuVVjfFoynPaVBUR8f+kN10EB/g',
        'sha256': '2d79fcc6b02a2e183a0cb30e0e25d103f42badda9fbf86bbee06f9'
                  '3aa3855aff',
        'type': 'PE32 executable (GUI) Intel 80386, for MS Windows',
        'id': 10,
        'md5': '9f2520a3056543d49bb0f822d85ce5dd',
        'size': 774144},)

    return response

  def _CreateTestEventObject(self, event_dictionary):
    """Create a test event with a set of attributes.

    Args:
      event_dictionary (dict[str, str]): contains attributes of an event to add
          to the queue.

    Returns:
      EventObject: event with the appropriate attributes for testing.
    """
    date_time = dfdatetime_posix_time.PosixTime(
        timestamp=event_dictionary['timestamp'])
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_CREATION)

    event.data_type = 'pe:compilation:compilation_time'
    event.pe_type = 'Executable (EXE)'

    for attribute_name, attribute_value in event_dictionary.items():
      if attribute_name == 'timestamp':
        continue

      setattr(event, attribute_name, attribute_value)

    return event

  def setUp(self):
    """Makes preparations before running an individual test."""
    self.requests_patcher = mock.patch('requests.post', self._MockPost)
    self.requests_patcher.start()

  def tearDown(self):
    """Cleans up after running an individual test."""
    self.requests_patcher.stop()

  def testExamineEventAndCompileReport(self):
    """Tests the ExamineEvent and CompileReport functions."""
    events = []
    for event_dictionary in self._TEST_EVENTS:
      event_dictionary['pathspec'] = fake_path_spec.FakePathSpec(
          location='C:\\WINDOWS\\system32\\evil.exe')

      event = self._CreateTestEventObject(event_dictionary)
      events.append(event)

    plugin = viper.ViperAnalysisPlugin()
    plugin.SetHost('localhost')
    plugin.SetPort(8080)
    plugin.SetProtocol('http')

    storage_writer = self._AnalyzeEvents(events, plugin)

    self.assertEqual(len(storage_writer.analysis_reports), 1)
    self.assertEqual(storage_writer.number_of_event_tags, 1)

    report = storage_writer.analysis_reports[0]
    self.assertIsNotNone(report)

    expected_text = (
        'viper hash tagging results\n'
        '1 path specifications tagged with label: viper_present\n'
        '1 path specifications tagged with label: viper_project_default\n'
        '1 path specifications tagged with label: viper_tag_darkcomet\n'
        '1 path specifications tagged with label: viper_tag_rat\n')

    self.assertEqual(report.text, expected_text)

    labels = []
    for event_tag in storage_writer.GetEventTags():
      labels.extend(event_tag.labels)
    self.assertEqual(len(labels), 4)

    expected_labels = [
        'viper_present', 'viper_project_default', 'viper_tag_darkcomet',
        'viper_tag_rat']
    self.assertEqual(sorted(labels), expected_labels)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Parser for Systemd journal files."""

from __future__ import unicode_literals

import os

import construct

from dfdatetime import posix_time as dfdatetime_posix_time

try:
  import lzma
except ImportError:
  from backports import lzma

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.lib import errors
from plaso.parsers import interface
from plaso.parsers import manager


class SystemdJournalEventData(events.EventData):
  """Systemd journal event data.

  Attributes:
    body (str): message body.
    hostname (str): hostname.
    pid (int): process identifier (PID).
    reporter (str): reporter.
  """

  DATA_TYPE = 'systemd:journal'

  def __init__(self):
    """Initializes event data."""
    super(SystemdJournalEventData, self).__init__(data_type=self.DATA_TYPE)
    self.body = None
    self.hostname = None
    self.pid = None
    self.reporter = None


class SystemdJournalParser(interface.FileObjectParser):
  """Parses Systemd Journal files."""

  NAME = 'systemd_journal'

  DESCRIPTION = 'Parser for Systemd Journal files.'

  _OBJECT_COMPRESSED_FLAG = 0x00000001

  # Unfortunately this doesn't help us knowing about the "dirtiness" or
  # "corrupted" file state.
  # A file can be in any of these states and still be corrupted, for example, by
  # an unexpected shut down. Once journald detects one of these, it will
  # "rotate" the corrupted journal file, an store it away, and change the status
  # to STATE_OFFLINE.
  # STATE_ONLINE means the file wasn't closed, but the journal can still be in a
  # clean state.
  _JOURNAL_STATE = construct.Enum(
      construct.ULInt8('state'),
      STATE_OFFLINE=0,
      STATE_ONLINE=1,
      STATE_ARCHIVED=2
  )

  _OBJECT_HEADER_TYPE = construct.Enum(
      construct.ULInt8('type'),
      UNUSED=0,
      DATA=1,
      FIELD=2,
      ENTRY=3,
      DATA_HASH_TABLE=4,
      FIELD_HASH_TABLE=5,
      ENTRY_ARRAY=6,
      TAG=7
  )

  _ULInt64 = construct.ULInt64('int')

  _OBJECT_HEADER = construct.Struct(
      'object_header',
      _OBJECT_HEADER_TYPE,
      construct.ULInt8('flags'),
      construct.Bytes('reserved', 6),
      construct.ULInt64('size')
  )

  _OBJECT_HEADER_SIZE = _OBJECT_HEADER.sizeof()

  _DATA_OBJECT = construct.Struct(
      'data_object',
      construct.ULInt64('hash'),
      construct.ULInt64('next_hash_offset'),
      construct.ULInt64('next_field_offset'),
      construct.ULInt64('entry_offset'),
      construct.ULInt64('entry_array_offset'),
      construct.ULInt64('n_entries')
  )

  _DATA_OBJECT_SIZE = _DATA_OBJECT.sizeof()

  _ENTRY_ITEM = construct.Struct(
      'entry_item',
      construct.ULInt64('object_offset'),
      construct.ULInt64('hash')
  )

  _ENTRY_OBJECT = construct.Struct(
      'entry_object',
      construct.ULInt64('seqnum'),
      construct.ULInt64('realtime'),
      construct.ULInt64('monotonic'),
      construct.Struct(
          'boot_id',
          construct.Bytes('bytes', 16),
          construct.ULInt64('qword1'),
          construct.ULInt64('qword2')),
      construct.ULInt64('xor_hash'),
      construct.Rename('object_items', construct.GreedyRange(_ENTRY_ITEM))
  )

  _JOURNAL_HEADER = construct.Struct(
      'journal_header',
      construct.Const(construct.String('signature', 8), b'LPKSHHRH'),
      construct.ULInt32('compatible_flags'),
      construct.ULInt32('incompatible_flags'),
      _JOURNAL_STATE,
      construct.Bytes('reserved', 7),
      construct.Bytes('file_id', 16),
      construct.Bytes('machine_id', 16),
      construct.Bytes('boot_id', 16),
      construct.Bytes('seqnum_id', 16),
      construct.ULInt64('header_size'),
      construct.ULInt64('arena_size'),
      construct.ULInt64('data_hash_table_offset'),
      construct.ULInt64('data_hash_table_size'),
      construct.ULInt64('field_hash_table_offset'),
      construct.ULInt64('field_hash_table_size'),
      construct.ULInt64('tail_object_offset'),
      construct.ULInt64('n_objects'),
      construct.ULInt64('n_entries'),
      construct.ULInt64('tail_entry_seqnum'),
      construct.ULInt64('head_entry_seqnum'),
      construct.ULInt64('entry_array_offset'),
      construct.ULInt64('head_entry_realtime'),
      construct.ULInt64('tail_entry_realtime'),
      construct.ULInt64('tail_entry_monotonic'),
      # Added in format version 187
      construct.ULInt64('n_data'),
      construct.ULInt64('n_fields'),
      # Added in format version 189
      construct.ULInt64('n_tags'),
      construct.ULInt64('n_entry_arrays')
  )

  def __init__(self):
    """Initializes a parser object."""
    super(SystemdJournalParser, self).__init__()
    self._max_journal_file_offset = 0

  def _ParseObjectHeader(self, file_object, offset):
    """Parses a Systemd journal object header structure.

    Args:
      file_object (dfvfs.FileIO): a file-like object.
      offset (int): offset to the object header.

    Returns:
      tuple[construct.Struct, int]: parsed object header and size of the
          object payload (data) that follows.
    """
    file_object.seek(offset, os.SEEK_SET)
    object_header_data = file_object.read(self._OBJECT_HEADER_SIZE)
    object_header = self._OBJECT_HEADER.parse(object_header_data)
    payload_size = object_header.size - self._OBJECT_HEADER_SIZE
    return (object_header, payload_size)

  def _ParseItem(self, file_object, offset):
    """Parses a Systemd journal DATA object.

    This method will read, and decompress if needed, the content of a DATA
    object.

    Args:
      file_object (dfvfs.FileIO): a file-like object.
      offset (int): offset to the DATA object.

    Returns:
      tuple[str, str]: key and value of this item.

    Raises:
      ParseError: When an unexpected object type is parsed.
    """
    object_header, payload_size = self._ParseObjectHeader(file_object, offset)
    file_object.read(self._DATA_OBJECT_SIZE)

    if object_header.type != 'DATA':
      raise errors.ParseError(
          'Expected an object of type DATA, but got {0:s}'.format(
              object_header.type))

    event_data = file_object.read(payload_size - self._DATA_OBJECT_SIZE)
    if object_header.flags & self._OBJECT_COMPRESSED_FLAG:
      event_data = lzma.decompress(event_data)

    event_string = event_data.decode('utf-8')
    event_key, event_value = event_string.split('=', 1)
    return (event_key, event_value)

  def _ParseJournalEntry(self, parser_mediator, file_object, offset):
    """Parses a Systemd journal ENTRY object.

    This method will generate an event per ENTRY object.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      file_object (dfvfs.FileIO): a file-like object.
      offset (int): offset of the ENTRY object.

    Raises:
      ParseError: When an unexpected object type is parsed.
    """
    object_header, payload_size = self._ParseObjectHeader(file_object, offset)

    entry_object_data = file_object.read(payload_size)
    entry_object = self._ENTRY_OBJECT.parse(entry_object_data)

    if object_header.type != 'ENTRY':
      raise errors.ParseError(
          'Expected an object of type ENTRY, but got {0:s}'.format(
              object_header.type))

    fields = {}
    for item in entry_object.object_items:
      if item.object_offset < self._max_journal_file_offset:
        raise errors.ParseError(
            'object offset should be after hash tables ({0:d} < {1:d})'.format(
                offset, self._max_journal_file_offset))
      key, value = self._ParseItem(file_object, item.object_offset)
      fields[key] = value

    reporter = fields.get('SYSLOG_IDENTIFIER', None)
    if reporter and reporter != 'kernel':
      pid = fields.get('_PID', fields.get('SYSLOG_PID', None))
    else:
      pid = None

    event_data = SystemdJournalEventData()
    event_data.body = fields['MESSAGE']
    event_data.hostname = fields['_HOSTNAME']
    event_data.pid = pid
    event_data.reporter = reporter

    date_time = dfdatetime_posix_time.PosixTimeInMicroseconds(
        timestamp=entry_object.realtime)
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_WRITTEN)
    parser_mediator.ProduceEventWithEventData(event, event_data)

  def _ParseEntries(self, file_object, offset):
    """Parses Systemd journal ENTRY_ARRAY objects.

    Args:
      file_object (dfvfs.FileIO): a file-like object.
      offset (int): offset of the ENTRY_ARRAY object.

    Returns:
      list[dict]: every ENTRY objects offsets.

    Raises:
      ParseError: When an unexpected object type is parsed.
    """
    entry_offsets = []
    object_header, payload_size = self._ParseObjectHeader(file_object, offset)

    if object_header.type != 'ENTRY_ARRAY':
      raise errors.ParseError(
          'Expected an object of type ENTRY_ARRAY, but got {0:s}'.format(
              object_header.type))

    next_array_offset = self._ULInt64.parse_stream(file_object)
    entry_offests_numbers, _ = divmod((payload_size - 8), 8)
    for entry_offset in range(entry_offests_numbers):
      entry_offset = self._ULInt64.parse_stream(file_object)
      if entry_offset != 0:
        entry_offsets.append(entry_offset)

    if next_array_offset != 0:
      next_entry_offsets = self._ParseEntries(file_object, next_array_offset)
      entry_offsets.extend(next_entry_offsets)

    return entry_offsets

  def ParseFileObject(self, parser_mediator, file_object, **kwargs):
    """Parses a Systemd journal file-like object.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      file_object (dfvfs.FileIO): a file-like object.

    Raises:
      UnableToParseFile: when the header cannot be parsed.
    """
    try:
      journal_header = self._JOURNAL_HEADER.parse_stream(file_object)
    except construct.ConstructError as exception:
      raise errors.UnableToParseFile(
          'Unable to parse journal header with error: {0!s}'.format(exception))

    max_data_hash_table_offset = (
        journal_header.data_hash_table_offset +
        journal_header.data_hash_table_size)
    max_field_hash_table_offset = (
        journal_header.field_hash_table_offset +
        journal_header.field_hash_table_size)
    self._max_journal_file_offset = max(
        max_data_hash_table_offset, max_field_hash_table_offset)

    entries_offsets = self._ParseEntries(
        file_object, journal_header.entry_array_offset)

    for entry_offset in entries_offsets:
      try:
        self._ParseJournalEntry(parser_mediator, file_object, entry_offset)
      except errors.ParseError as exception:
        parser_mediator.ProduceExtractionError((
            'Unable to complete parsing journal file: {0:s} at offset '
            '0x{1:08x}').format(exception, entry_offset))
        return
      except construct.ConstructError as exception:
        raise errors.UnableToParseFile((
            'Unable to parse journal header at offset: 0x{0:08x} with '
            'error: {1:s}').format(entry_offset, exception))


manager.ParsersManager.RegisterParser(SystemdJournalParser)
